import * as Tp from 'thingpedia';
import { FormattedObject } from './formatter';
import TwilioNotificationBackend from './twilio';
import EmailNotificationBackend from './email';
export interface NotificationDelegate {
    notify(data: {
        appId: string;
        icon: string | null;
        raw: Record<string, unknown>;
        type: string;
        formatted: FormattedObject[];
    }): Promise<void>;
    notifyError(data: {
        appId: string;
        icon: string | null;
        error: Error;
    }): Promise<void>;
}
export interface NotificationBackend {
    readonly name: string;
    readonly uniqueId: string;
    readonly requiredSettings: Record<string, string>;
    notify(data: {
        appId: string;
        icon: string | null;
        raw: Record<string, unknown>;
        type: string;
        formatted: FormattedObject[];
    }, config?: Record<string, string>): Promise<void>;
    notifyError(data: {
        appId: string;
        icon: string | null;
        error: Error;
    }, config?: Record<string, string>): Promise<void>;
}
export declare const StaticNotificationBackends: {
    twilio: typeof TwilioNotificationBackend;
    email: typeof EmailNotificationBackend;
};
export declare type NotificationConfig = {
    [T in keyof typeof StaticNotificationBackends]?: ConstructorParameters<(typeof StaticNotificationBackends)[T]>[1];
};
/**
 * Helper class to adapt a Thingpedia device into a notification backend.
 */
export declare class ThingpediaNotificationBackend implements NotificationBackend {
    private _iface;
    name: string;
    uniqueId: string;
    constructor(device: Tp.BaseDevice);
    get requiredSettings(): {};
    notify(data: {
        appId: string;
        icon: string | null;
        raw: Record<string, unknown>;
        type: string;
        formatted: FormattedObject[];
    }): Promise<void>;
    notifyError(data: {
        appId: string;
        icon: string | null;
        error: Error;
    }): Promise<void>;
}
