/// <reference types="node" />
import * as events from 'events';
import * as Tp from 'thingpedia';
import type DeviceDatabase from './devices/database';
/**
 * Coordinate access to audio between multiple skills.
 */
export default class AudioController extends events.EventEmitter {
    private _devices;
    private _deviceRemovedListener;
    private _currentDevice;
    private _releaseCallback;
    constructor(devices: DeviceDatabase);
    start(): Promise<void>;
    stop(): Promise<void>;
    /**
     * Request background audio on behalf of the given device.
     *
     * This method must be called before starting a background playback operation,
     * using a media-player capability or using a device-specific service.
     *
     * The provided release callback will be called when a different device
     * request audio at a later time. When called, the release callback must stop
     * playback on behalf of the current device.
     *
     * This method can be called multiple times for the same device, with no effect.
     */
    requestAudio(device: Tp.BaseDevice, releaseCallback: () => void | Promise<void>): Promise<void>;
    /**
     * Request background audio on behalf of the system itself.
     *
     * This method must not be called outside of Genie
     */
    requestSystemAudio(releaseCallback: () => void | Promise<void>): Promise<void>;
    /**
     * Release the control of background audio on behalf of the given device.
     *
     * This method has no effect if the given device is not currently controlling
     * audio.
     */
    releaseAudio(device: Tp.BaseDevice): Promise<void>;
    /**
     * Stop all audio coming from this assistant.
     *
     * This method will inform the currently playing device that it must stop
     * playing. It corresponds to the command "stop".
     */
    stopAudio(): Promise<void>;
    private _onDeviceRemoved;
}
