"use strict";
// -*- mode: typescript; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2021 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const events = __importStar(require("events"));
/**
 * Coordinate access to audio between multiple skills.
 */
class AudioController extends events.EventEmitter {
    constructor(devices) {
        super();
        this._currentDevice = null;
        this._releaseCallback = null;
        this._devices = devices;
        this._deviceRemovedListener = this._onDeviceRemoved.bind(this);
    }
    async start() {
        this._devices.on('device-removed', this._deviceRemovedListener);
    }
    async stop() {
        this._devices.removeListener('device-removed', this._deviceRemovedListener);
        await this.stopAudio();
    }
    /**
     * Request background audio on behalf of the given device.
     *
     * This method must be called before starting a background playback operation,
     * using a media-player capability or using a device-specific service.
     *
     * The provided release callback will be called when a different device
     * request audio at a later time. When called, the release callback must stop
     * playback on behalf of the current device.
     *
     * This method can be called multiple times for the same device, with no effect.
     */
    async requestAudio(device, releaseCallback) {
        if (device === this._currentDevice) {
            this._releaseCallback = releaseCallback;
            return;
        }
        if (this._releaseCallback)
            this._releaseCallback();
        this._currentDevice = device;
        console.log(`Switching audio to ${this._currentDevice.uniqueId}`);
        this._releaseCallback = releaseCallback;
    }
    /**
     * Request background audio on behalf of the system itself.
     *
     * This method must not be called outside of Genie
     */
    async requestSystemAudio(releaseCallback) {
        if (this._releaseCallback)
            this._releaseCallback();
        this._currentDevice = null;
        console.log(`Switching audio to system`);
        this._releaseCallback = releaseCallback;
    }
    /**
     * Release the control of background audio on behalf of the given device.
     *
     * This method has no effect if the given device is not currently controlling
     * audio.
     */
    async releaseAudio(device) {
        if (device !== this._currentDevice)
            return;
        this._currentDevice = null;
        this._releaseCallback = null;
    }
    /**
     * Stop all audio coming from this assistant.
     *
     * This method will inform the currently playing device that it must stop
     * playing. It corresponds to the command "stop".
     */
    async stopAudio() {
        this.emit('stop');
        if (this._releaseCallback)
            await this._releaseCallback();
        this._releaseCallback = null;
    }
    _onDeviceRemoved(device) {
        if (device === this._currentDevice) {
            console.log(`Audio device removed`);
            this._currentDevice = null;
            this._releaseCallback = null;
        }
    }
}
exports.default = AudioController;
//# sourceMappingURL=audio_controller.js.map