"use strict";
// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Mehrad Moradshahi <mehrad@cs.stanford.edu>
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const default_1 = __importDefault(require("./default"));
class PolishLanguagePack extends default_1.default {
    constructor(locale) {
        super(locale);
    }
    isGoodWord(word) {
        // filter out words that cannot be in the dataset,
        // because they would be either tokenized/preprocessed out or
        // they are unlikely to be used with voice
        return /^([aąbcćdeęfghijklłmnńoóprsśtuwyzźż0-9][aąbcćdeęfghijklłmnńoóprsśtuwyzźż0-9.-]*|\u060C|\u061F)$/.test(word);
    }
    isGoodSentence(sentence) {
        if (sentence.length < 3)
            return false;
        if (['.', '\u060C', '\u061F', '!', ' '].includes(sentence[0]))
            return false;
        // (for|me|and|or|that|this|in|with|from|on|before|after)$
        return !/^(dla|mnie|i|lub|że|ten|in|z|od|on|przed|po)$/.test(sentence);
    }
    isGoodNumber(number) {
        // [English numbers| Persian numbers]
        return /^([A-Za-z]*[0-9|\u0660-\u0669]+)$/.test(number);
    }
    isGoodPersonName(word) {
        return this.isGoodWord(word) || /^([aąbcćdeęfghijklłmnńoóprsśtuwyzźż]+\s[aąbcćdeęfghijklłmnńoóprsśtuwyzźż]+\s?\.?)$/.test(word);
    }
}
exports.default = PolishLanguagePack;
//# sourceMappingURL=polish.js.map