"use strict";
// -*- mode: typescript; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPrimitiveIcon = exports.getProgramIcon = void 0;
const thingtalk_1 = require("thingtalk");
function isPlatformBuiltin(kind) {
    return kind.startsWith('org.thingpedia.builtin.thingengine');
}
function getProgramIcon(program) {
    let icon = null;
    let node = program;
    if (node instanceof thingtalk_1.Ast.DialogueState) {
        const last = node.history[node.history.length - 1];
        if (!last)
            return null;
        node = last;
    }
    for (const [, prim] of node.iteratePrimitives(false)) {
        if (!(prim.selector instanceof thingtalk_1.Ast.DeviceSelector))
            continue;
        const newIcon = getPrimitiveIcon(prim);
        // ignore builtin/platform devices when choosing the icon
        if (!newIcon || isPlatformBuiltin(newIcon))
            continue;
        icon = newIcon;
    }
    return icon;
}
exports.getProgramIcon = getProgramIcon;
function getPrimitiveIcon(prim) {
    if (prim === null)
        return null;
    let kind = null;
    if (prim instanceof thingtalk_1.Ast.SpecifiedPermissionFunction)
        kind = prim.kind;
    else if (prim.selector instanceof thingtalk_1.Ast.DeviceSelector)
        kind = prim.selector.kind;
    return kind;
}
exports.getPrimitiveIcon = getPrimitiveIcon;
//# sourceMappingURL=icons.js.map