/**
 * Covert wikidata unit into thingtalk unit
 * @param wikidataUnit
 */
export function unitConverter(wikidataUnit: any): any;
/**
 * Run SPARQL query on wikidata endpoint and retrieve results
 * @param {string} query: SPARQL query
 * @returns {Promise<*>}
 */
export function wikidataQuery(query: string): Promise<any>;
/**
 * Get the label of a given property
 * @param {string} propertyId: the id of the property
 * @returns {Promise<null|string>}: the label of the property
 */
export function getPropertyLabel(propertyId: string): Promise<null | string>;
/**
 * Get alternative labels of a given property
 * @param {string} propertyId: the id of the property
 * @returns {Promise<Array.string>}: the label of the property
 */
export function getPropertyAltLabels(propertyId: string): Promise<any>;
/**
 * Get the label of a given item
 * @param {string} itemId: the id of the item
 * @returns {Promise<null|string>}: the label of the item
 */
export function getItemLabel(itemId: string): Promise<null | string>;
/**
 * Get a list of common properties given a domain
 * @param {string} domainId: the id of the domain, e.g. "Q5" for human domain
 * @returns {Promise<Array.string>}: a list of property ids
 */
export function getPropertyList(domainId: string): Promise<any>;
/**
 * Get the value type constraint (Q21510865) of a property
 * @param propertyId
 * @returns {Promise<Array.Object<id,value>>} A list of allowed value types and their labels
 */
export function getValueTypeConstraint(propertyId: any): Promise<any>;
/**
 * Get the one-of constraint (Q21510859) of a property
 * This allows to detect Enum types
 *
 * @param propertyId
 * @returns {Promise<Array.String>} A list of enum values
 */
export function getOneOfConstraint(propertyId: any): Promise<any>;
/**
 * Get the allowed units (Q21514353) of a property
 * This allows to detect Measure types
 *
 * @param propertyId
 * @returns {Promise<Array.String>} A list of allowed units
 */
export function getAllowedUnits(propertyId: any): Promise<any>;
/**
 * Get the allowed units (Q21514353) of a property
 * This allows to detect Measure types
 *
 * @param propertyId
 * @returns {Promise<Array.Object<max, min>>|null} A list of allowed units
 */
export function getRangeConstraint(propertyId: any): Promise<any> | null;
/**
 * Get the Schema.org equivalent given a wikidata property
 * @param {string} propertyId: the id of a property
 * @returns {Promise<string>}: name of the equivalent property in Schema.org
 */
export function getSchemaorgEquivalent(propertyId: string): Promise<string>;
/**
 * Get the class (instance of) of a given wikidata property or entity
 * @param {string} id: the id of a property or an entity
 * @returns {Promise<Array.string>}: list of classes
 */
export function getClasses(id: string): Promise<any>;
/**
 * Get wikidata equivalent of a given wikidata property or entity
 * @param {string} id: the id of a property or an entity
 * @returns {Promise<Array.string>}: list of classes
 */
export function getEquivalent(id: string): Promise<any>;
