"use strict";
// -*- mode: typescript; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Tp = __importStar(require("thingpedia"));
const file_parameter_provider_1 = __importDefault(require("./file_parameter_provider"));
/**
 * A subclass of {@link thingpedia}.FileClient that supports
 * looking up entities by value using the file parameter provider.
 */
class FileThingpediaClient extends Tp.FileClient {
    constructor(options) {
        super(options);
        this._cachedEntities = new Map;
        this._provider = new file_parameter_provider_1.default(options.parameter_datasets, options.locale);
    }
    async lookupEntity(entityType, searchTerm) {
        // ignore search term, return everything
        const cached = this._cachedEntities.get(entityType);
        if (cached)
            return { data: cached, meta: { name: entityType, is_well_known: false, has_ner_support: true } };
        const result = await this._provider.getEntity(entityType);
        this._cachedEntities.set(entityType, result);
        return { data: result, meta: { "name": entityType, is_well_known: false, has_ner_support: true } };
    }
}
exports.default = FileThingpediaClient;
//# sourceMappingURL=file_thingpedia_client.js.map