"use strict";
// -*- mode: typescript; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeOtherAggregationReply = exports.makeCountAggregationReply = void 0;
const thingtalk_1 = require("thingtalk");
const C = __importStar(require("../ast_manip"));
const state_manip_1 = require("../state_manip");
function makeCountAggregationReply(ctx, table, mustFilter) {
    if (!ctx.resultInfo.isAggregation)
        return null;
    const results = ctx.results;
    if (!results || results.length !== 1 || !results[0].value.count)
        return null;
    const currentStmt = ctx.current.stmt;
    const currentTable = currentStmt.lastQuery;
    if (!(currentTable instanceof thingtalk_1.Ast.AggregationExpression) ||
        currentTable.operator !== 'count' || currentTable.field !== '*')
        return null;
    if (!C.isSameFunction(table.schema, ctx.currentTableFunction))
        return null;
    if (mustFilter && !(table instanceof thingtalk_1.Ast.FilterExpression))
        return null;
    if (table instanceof thingtalk_1.Ast.FilterExpression) {
        const filterTable = C.findFilterExpression(currentTable);
        if (!filterTable)
            return null;
        if (!table.filter.equals(filterTable.filter))
            return null;
    }
    return state_manip_1.makeAgentReply(ctx, state_manip_1.makeSimpleState(ctx, 'sys_display_result', null));
}
exports.makeCountAggregationReply = makeCountAggregationReply;
function makeOtherAggregationReply(ctx, op, param, value) {
    if (!ctx.resultInfo.isAggregation)
        return null;
    const results = ctx.results;
    if (!results || results.length !== 1 || !results[0].value[param.name])
        return null;
    const currentStmt = ctx.current.stmt;
    const currentTable = currentStmt.lastQuery;
    if (!(currentTable instanceof thingtalk_1.Ast.AggregationExpression) ||
        currentTable.operator !== op ||
        !C.isSameFunction(currentTable.schema, param.schema) ||
        currentTable.field !== param.name)
        return null;
    if (!value.equals(results[0].value[param.name]))
        return null;
    return state_manip_1.makeAgentReply(ctx, state_manip_1.makeSimpleState(ctx, 'sys_display_result', null));
}
exports.makeOtherAggregationReply = makeOtherAggregationReply;
//# sourceMappingURL=aggregation-results.js.map