// -*- mode: typescript; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

import assert from 'assert';
import { Ast, } from 'thingtalk';
import { SentenceGeneratorTypes } from 'genie-toolkit';
import * as C from '../ast_manip';
import * as S from '../state_manip';
import * as D from '../dialogue_acts';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {

$loader = new ThingpediaLoader($runtime, $ttUtils, $grammar, $locale, $options);
   await (await $runtime.import("../common.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);

await $loader.init();
   await (await $runtime.import("./shared.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./constants.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./timers.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./filters.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./projections.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./aggregation.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./computation.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./who_questions.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./stream_tables.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./commands.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/shared.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/results.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/coref-actions.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/action-confirm.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/action-results.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/aggregation-results.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/coref-questions.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/empty-search.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/initial-request.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/list-proposal.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/recommendation.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/refinement.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/related-questions.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/search-questions.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/slot-fill.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./dlg/streams.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   $grammar.declareContext("ctx_init");
   $grammar.declareContext("ctx_multidomain");
   $grammar.declareContext("ctx_with_result");
   $grammar.declareContext("ctx_with_table_result");
   $grammar.declareContext("ctx_with_aggregation_result");
   $grammar.declareContext("ctx_with_action");
   $grammar.declareContext("ctx_without_action");
   $grammar.declareContext("ctx_with_result_and_action");
   $grammar.declareContext("ctx_with_result_question");
   $grammar.declareContext("ctx_with_result_noquestion");
   $grammar.declareContext("ctx_with_result_argminmax");
   $grammar.declareContext("ctx_without_projection");
   $grammar.declareContext("ctx_with_notification");
   $grammar.declareContext("ctx_for_related_question");
   $grammar.declareContext("ctx_next_statement");
   $grammar.declareContext("ctx_current_statement");
   $grammar.declareContext("ctx_greet");
   $grammar.declareContext("ctx_reinit");
   $grammar.declareContext("ctx_cancel");
   $grammar.declareContext("ctx_end");
   $grammar.declareContext("ctx_search_command");
   $grammar.declareContext("ctx_aggregation_question");
   $grammar.declareContext("ctx_distance_argminmax_question");
   $grammar.declareContext("ctx_argminmax_question");
   $grammar.declareContext("ctx_one_result_question");
   $grammar.declareContext("ctx_empty_direct_question");
   $grammar.declareContext("ctx_empty_search_command");
   $grammar.declareContext("ctx_large_search_command");
   $grammar.declareContext("ctx_single_result_search_command");
   $grammar.declareContext("ctx_complete_search_command");
   $grammar.declareContext("ctx_completed_action_success");
   $grammar.declareContext("ctx_completed_action_error");
   $grammar.declareContext("ctx_confirm_action");
   $grammar.declareContext("ctx_incomplete_action");
   $grammar.declareContext("ctx_incomplete_action_after_search");
   $grammar.declareContext("ctx_learn_more");
   $grammar.declareContext("ctx_display_nonlist_result");
   $grammar.declareContext("ctx_rule_enable_success");
   $grammar.declareContext("ctx_action_notification");
   $grammar.declareContext("ctx_list_notification");
   $grammar.declareContext("ctx_nonlist_notification");
   $grammar.declareContext("ctx_notification_error");
   $grammar.declareContext("ctx_sys_any");
   $grammar.declareContext("ctx_sys_action_success");
   $grammar.declareContext("ctx_sys_action_error");
   $grammar.declareContext("ctx_sys_anything_else");
   $grammar.declareContext("ctx_sys_confirm_action");
   $grammar.declareContext("ctx_sys_display_result");
   $grammar.declareContext("ctx_sys_empty_search");
   $grammar.declareContext("ctx_sys_end");
   $grammar.declareContext("ctx_sys_greet");
   $grammar.declareContext("ctx_sys_learn_more_what");
   $grammar.declareContext("ctx_sys_propose_refined_query");
   $grammar.declareContext("ctx_sys_recommend_many");
   $grammar.declareContext("ctx_sys_recommend_one");
   $grammar.declareContext("ctx_sys_search_question");
   $grammar.declareContext("ctx_sys_slot_fill");
   $grammar.declareContext("ctx_current_query");
   $grammar.declareContext("ctx_next_query");
   $grammar.declareContext("ctx_next_action");
   $grammar.declareContext("ctx_thingpedia_result");
   $grammar.declareContext("ctx_thingpedia_list_result");
   $grammar.declareContext("ctx_thingpedia_error_message");
   $grammar.declareContext("ctx_result_name_list");
   $grammar.declareContext("ctx_result_name");
   $grammar.declareContext("ctx_notification_app_name");
   $grammar.declareFunction('context', (state : Ast.DialogueState, contextTable : SentenceGeneratorTypes.ContextTable) : SentenceGeneratorTypes.ContextPhrase[]|null => {
    if (state === null)
        return [S.makeContextPhrase(contextTable.ctx_init, S.initialContextInfo($loader, contextTable))];

    assert(state instanceof Ast.DialogueState, `expected a dialogue state Ast node`);
    if (state.policy !== S.POLICY_NAME)
        return null;
    (global as any).enableBreakpoint = true;
    const ctx = S.getContextInfo($loader, state, contextTable);
    (global as any).enableBreakpoint = false;
    // get the main context tags for this context (the abstract state, in paper terminology)
    const tags = S.tagContextForAgent(ctx);

    const phrases = tags.map((tag) => S.makeContextPhrase(tag, ctx));
    // add auxiliary context non-terminals used for pruning and to simplify generation
    phrases.push(...S.getContextPhrases(ctx));
    return phrases;
});
   $grammar.declareFunction('answer', (state : Ast.DialogueState, answer : Ast.Value, contextTable : SentenceGeneratorTypes.ContextTable) : Ast.DialogueState|null => {
    const ctx = S.getContextInfo($loader, state, contextTable);

    // if the agent proposed something and the user says "yes", we accept the proposal
    if (state.history.length > 0 && state.history[state.history.length-1].confirm === 'proposed'
        && answer instanceof Ast.BooleanValue) {
        if (answer.value) // yes accepts
            return S.acceptAllProposedStatements(ctx);
        else // no is cancel
            return S.makeSimpleState(ctx, 'cancel', null);
    }

    switch (state.dialogueAct) {
    case 'sys_record_command':
        return S.makeSimpleState(ctx, 'end', null);

    case 'sys_anything_else':
        if (answer instanceof Ast.BooleanValue) {
            if (answer.value)
                return S.makeSimpleState(ctx, 'reinit', null);
            else
                return S.makeSimpleState(ctx, 'end', null);
        }
        return null;
    case 'sys_recommend_one':
    case 'sys_recommend_two':
    case 'sys_recommend_three':
    case 'sys_recommend_four':
        // "yes" to a recommendation (without a proposed action) is an answer to
        // "would you like to learn more"
        if (answer instanceof Ast.BooleanValue && answer.value === true)
            return S.makeSimpleState(ctx, 'learn_more', null);
        // fallthrough
    case 'sys_display_result':
        // "no" to a recommendation or display result is cancel
        if (answer instanceof Ast.BooleanValue && answer.value === false)
            return S.makeSimpleState(ctx, 'cancel', null);
        return null;

    case 'sys_slot_fill':
        return D.impreciseSlotFillAnswer(ctx, answer);
    case 'sys_search_question':
        return D.impreciseSearchQuestionAnswer(ctx, answer);
    case 'sys_confirm_action':
        if (answer instanceof Ast.BooleanValue) {
            if (answer.value)
                return D.actionConfirmAcceptPhrase(ctx);
            else
                return D.actionConfirmRejectPhrase(ctx);
        }
        return null;
    default:
        return null;
    }
});
   $grammar.declareFunction('notification', (appName : string|null, program : Ast.Program, result : Ast.DialogueHistoryResultItem) : any => {
    assert(program.statements.length === 1);
    const stmt = program.statements[0];
    assert(stmt instanceof Ast.ExpressionStatement);

    return new Ast.DialogueState(null, S.POLICY_NAME, 'notification', appName ? [new Ast.Value.String(appName)] : null,
        [new Ast.DialogueHistoryItem(null, stmt, new Ast.DialogueHistoryResultList(null, [result], new Ast.NumberValue(1), false), 'confirmed')]);
});
   $grammar.declareFunction('notifyError', (appName : string|null, program : Ast.Program, error : Ast.Value) : any => {
    assert(program.statements.length === 1);
    const stmt = program.statements[0];
    assert(stmt instanceof Ast.ExpressionStatement);

    return new Ast.DialogueState(null, S.POLICY_NAME, 'notification', appName ? [new Ast.Value.String(appName)] : null,
        [new Ast.DialogueHistoryItem(null, stmt, new Ast.DialogueHistoryResultList(null, [], new Ast.NumberValue(1), false, error), 'confirmed')]);
});
   $grammar.declareSymbol("$agent");
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_greet", "ctx", undefined)], $locale._("${ctx} {hello|hi} {!|,} {how can i help you|what are you interested in|what can i do for you} ?"), ((ctx : S.ContextInfo) : (S.AgentReplyRecord)|null => S.makeAgentReply(ctx, S.makeSimpleState(ctx, 'sys_greet', null), null, null, { end: false })), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_init", "ctx", undefined)], $locale._("${ctx} {hello|hi} {!|,} {how can i help you|what are you interested in|what can i do for you} ?"), ((ctx : S.ContextInfo) : (S.AgentReplyRecord)|null => S.makeAgentReply(ctx, S.makeSimpleState(ctx, 'sys_greet', null), null, null, { end: false })), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_reinit", "ctx", undefined)], $locale._("${ctx} {how can i help you|what are you interested in|what can i do for you} ?"), ((ctx : S.ContextInfo) : (S.AgentReplyRecord)|null => S.makeAgentReply(ctx, S.makeSimpleState(ctx, 'sys_greet', null), null, null, { end: false })), undefined, {  });
   if ($options.flags.anything_else) {
       $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_cancel", "ctx", undefined), new $runtime.NonTerminal("anything_else_phrase", undefined, undefined)], $locale._("${ctx} ${anything_else_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (S.AgentReplyRecord)|null => S.makeAgentReply(ctx, S.makeSimpleState(ctx, 'sys_anything_else', null), null, null, { end: false })), undefined, {  });
       $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_end", "ctx", undefined)], $locale._("${ctx} alright , {bye !|good bye !}"), ((ctx : S.ContextInfo) : (S.AgentReplyRecord)|null => S.makeAgentReply(ctx, S.makeSimpleState(ctx, 'sys_end', null))), undefined, {  });
   }
   if (!$options.flags.anything_else) {
       $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_cancel", "ctx", undefined)], $locale._("${ctx} alright, let me know if I can help you with anything else!"), ((ctx : S.ContextInfo) : (S.AgentReplyRecord)|null => S.makeAgentReply(ctx, S.makeSimpleState(ctx, 'sys_end', null))), undefined, {  });
   }
   $grammar.addRule("$agent", [new $runtime.NonTerminal("action_success_phrase", undefined, undefined)], "${action_success_phrase}", (($0 : S.AgentReplyRecord) : (S.AgentReplyRecord)|null => { return $0; }), undefined, { identity: (true) });
   if ($options.flags.anything_else) {
       $grammar.addRule("$agent", [new $runtime.NonTerminal("action_success_phrase", "action", undefined), new $runtime.NonTerminal("anything_else_phrase", undefined, undefined)], $locale._("${action} ${anything_else_phrase}"), ((action : S.AgentReplyRecord, $0 : string) : (S.AgentReplyRecord)|null => S.setEndBit(action, false)), undefined, {  });
   }
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_completed_action_error", "ctx", undefined), new $runtime.NonTerminal("sorry_preamble", undefined, undefined), new $runtime.NonTerminal("action_error_phrase", "phrase", undefined)], $locale._("${ctx} ${sorry_preamble} ${phrase}"), ((ctx : S.ContextInfo, $0 : string, phrase : S.AgentReplyRecord) : (S.AgentReplyRecord)|null => phrase), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_display_nonlist_result", "ctx", undefined), new $runtime.NonTerminal("system_nonlist_result", "proposal", undefined)], $locale._("${ctx} ${proposal}"), ((ctx : S.ContextInfo, proposal : D.Recommendation) : (S.AgentReplyRecord)|null => D.makeDisplayResultReply(ctx, proposal)), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_display_nonlist_result", "ctx", undefined), new $runtime.NonTerminal("system_list_proposal", "proposal", undefined)], $locale._("${ctx} ${proposal}"), ((ctx : S.ContextInfo, proposal : D.ListProposal) : (S.AgentReplyRecord)|null => D.makeDisplayResultReplyFromList(ctx, proposal)), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_aggregation_question", "ctx", undefined), new $runtime.NonTerminal("count_aggregation_reply", "reply", undefined)], $locale._("${ctx} ${reply}"), ((ctx : S.ContextInfo, reply : S.AgentReplyRecord) : (S.AgentReplyRecord)|null => reply), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_aggregation_question", "ctx", undefined), new $runtime.NonTerminal("other_aggregation_reply", "reply", undefined)], $locale._("${ctx} ${reply}"), ((ctx : S.ContextInfo, reply : S.AgentReplyRecord) : (S.AgentReplyRecord)|null => reply), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_confirm_action", "ctx", undefined), new $runtime.NonTerminal("action_confirm_phrase", "reply", undefined)], $locale._("${ctx} ${reply}"), ((ctx : S.ContextInfo, reply : S.AgentReplyRecord) : (S.AgentReplyRecord)|null => reply), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_empty_search_command", "ctx", undefined), new $runtime.NonTerminal("empty_search_error", "error", undefined)], $locale._("${ctx} ${error}"), ((ctx : S.ContextInfo, error : [Ast.InvocationExpression|null, C.ParamSlot|null, boolean]) : (S.AgentReplyRecord)|null => D.makeEmptySearchError(ctx, error)), undefined, {  });
   if (!$options.flags.inference) {
       $grammar.addRule("$agent", [new $runtime.NonTerminal("system_generic_proposal", "prop", undefined)], "${prop}", ((prop : S.AgentReplyRecord) : (S.AgentReplyRecord)|null => prop), undefined, {  });
   }
   if (!$options.flags.inference) {
       $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_search_command", "ctx", undefined), new $runtime.NonTerminal("search_question", "questions", undefined)], $locale._("${ctx} ${questions}"), ((ctx : S.ContextInfo, questions : C.ParamSlot[]) : (S.AgentReplyRecord)|null => D.makeSearchQuestion(ctx, questions)), undefined, {  });
   }
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_incomplete_action_after_search", "ctx", undefined), new $runtime.NonTerminal("slot_fill_question", "questions", undefined)], $locale._("${ctx} ${questions}"), ((ctx : S.ContextInfo, questions : C.ParamSlot[]) : (S.AgentReplyRecord)|null => D.makeSlotFillQuestion(ctx, questions)), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_complete_search_command", "ctx", undefined), new $runtime.NonTerminal("system_recommendation", "proposal", undefined)], $locale._("${ctx} ${proposal}"), ((ctx : S.ContextInfo, proposal : D.Recommendation) : (S.AgentReplyRecord)|null => D.makeRecommendationReply(ctx, proposal)), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_complete_search_command", "ctx", undefined), new $runtime.NonTerminal("system_list_proposal", "proposal", undefined)], $locale._("${ctx} ${proposal}"), ((ctx : S.ContextInfo, proposal : D.ListProposal) : (S.AgentReplyRecord)|null => D.makeListProposalReply(ctx, proposal)), undefined, { weight: (10) });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_learn_more", "ctx", undefined), new $runtime.NonTerminal("system_learn_more", undefined, undefined)], $locale._("${ctx} ${system_learn_more}"), ((ctx : S.ContextInfo, $0 : string) : (S.AgentReplyRecord)|null => S.makeAgentReply(ctx, S.makeSimpleState(ctx, 'sys_learn_more_what', null), null, null, { end: false })), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_rule_enable_success", "ctx", undefined), new $runtime.NonTerminal("generic_excitement2_phrase", "preamble", undefined), new $runtime.NonTerminal("ctx_current_statement", "stmt", undefined)], $locale._("${preamble} I {will|am going to} ${stmt}"), ((ctx : S.ContextInfo, preamble : string, stmt : S.ContextInfo) : (S.AgentReplyRecord)|null => S.makeAgentReply(ctx, S.makeSimpleState(ctx, 'sys_rule_enable_success', null))), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_list_notification", "ctx", undefined), new $runtime.NonTerminal("notification_preamble", undefined, undefined), new $runtime.NonTerminal("system_recommendation", "proposal", undefined)], $locale._("${ctx} ${notification_preamble} ${proposal}"), ((ctx : S.ContextInfo, $0 : S.ContextInfo, proposal : D.Recommendation) : (S.AgentReplyRecord)|null => D.makeRecommendationReply(ctx, proposal)), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_nonlist_notification", "ctx", undefined), new $runtime.NonTerminal("notification_preamble", undefined, undefined), new $runtime.NonTerminal("system_nonlist_result", "proposal", undefined)], $locale._("${ctx} ${notification_preamble} ${proposal}"), ((ctx : S.ContextInfo, $0 : S.ContextInfo, proposal : D.Recommendation) : (S.AgentReplyRecord)|null => D.makeDisplayResultReply(ctx, proposal)), undefined, {  });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("action_notification_phrase", undefined, undefined)], "${action_notification_phrase}", (($0 : S.AgentReplyRecord) : (S.AgentReplyRecord)|null => { return $0; }), undefined, { identity: (true) });
   $grammar.addRule("$agent", [new $runtime.NonTerminal("ctx_notification_error", "ctx", undefined), new $runtime.NonTerminal("notification_error_preamble", undefined, undefined), new $runtime.NonTerminal("action_error_phrase", "phrase", undefined)], $locale._("${ctx} ${notification_error_preamble} ${phrase}"), ((ctx : S.ContextInfo, $0 : S.ContextInfo, phrase : S.AgentReplyRecord) : (S.AgentReplyRecord)|null => phrase), undefined, {  });
   $grammar.declareSymbol("$user");
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_init", "ctx", undefined), new $runtime.NonTerminal("greeting", undefined, undefined)], $locale._("${ctx} ${greeting}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'greet', null)), undefined, { weight: (0.01), expandchoice: (false) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_init", "ctx", undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${stmt}"), ((ctx : S.ContextInfo, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, { weight: (0.5) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_init", "ctx", undefined), new $runtime.NonTerminal("greeting", undefined, undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${greeting} ${stmt}"), ((ctx : S.ContextInfo, $0 : string, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, { weight: (0.5) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_greet", "ctx", undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${stmt}"), ((ctx : S.ContextInfo, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_anything_else", "ctx", undefined), new $runtime.NonTerminal("no_thanks_phrase", undefined, undefined)], $locale._("${ctx} ${no_thanks_phrase} {goodbye !|}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'end', null)), undefined, { weight: (0.01) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_anything_else", "ctx", undefined), new $runtime.NonTerminal("thanks_phrase", undefined, undefined)], $locale._("${ctx} ${thanks_phrase} {goodbye !|}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'end', null)), undefined, { weight: (0.01) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_anything_else", "ctx", undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${stmt}"), ((ctx : S.ContextInfo, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_anything_else", "ctx", undefined), new $runtime.NonTerminal("yes_please_phrase", undefined, undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${yes_please_phrase} ${stmt}"), ((ctx : S.ContextInfo, $0 : string, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_anything_else", "ctx", undefined), new $runtime.NonTerminal("yes_please_phrase", undefined, undefined)], $locale._("${ctx} ${yes_please_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'reinit', null)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_confirm_action", "ctx", undefined), new $runtime.NonTerminal("accept_phrase", undefined, undefined)], $locale._("${ctx} ${accept_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.actionConfirmAcceptPhrase(ctx)), undefined, { weight: (0.9) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_confirm_action", "ctx", undefined), new $runtime.NonTerminal("reject_phrase", undefined, undefined)], $locale._("${ctx} ${reject_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.actionConfirmRejectPhrase(ctx)), undefined, { weight: (0.1) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_confirm_action", "ctx", undefined), new $runtime.NonTerminal("reject_phrase", undefined, undefined), new $runtime.NonTerminal("imprecise_slot_fill_answer_phrase", "answer", undefined)], $locale._("${ctx} ${reject_phrase} {can you try|how about} ${answer} {instead ?|}"), ((ctx : S.ContextInfo, $0 : string, answer : Ast.Value|C.InputParamSlot) : (Ast.DialogueState)|null => D.actionConfirmChangeParam(ctx, answer)), undefined, { weight: (0.1) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_action_success", "ctx", undefined), new $runtime.NonTerminal("thanks_phrase", undefined, undefined)], $locale._("${ctx} ${thanks_phrase} {goodbye !|}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'cancel', null)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_display_result", "ctx", undefined), new $runtime.NonTerminal("thanks_phrase", undefined, undefined)], $locale._("${ctx} ${thanks_phrase} {goodbye !|}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'cancel', null)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_action_success", "ctx", undefined), new $runtime.NonTerminal("simple_user_search_question", "questions", undefined)], $locale._("${ctx} ${questions}"), ((ctx : S.ContextInfo, questions : C.ParamSlot[]) : (Ast.DialogueState)|null => D.actionSuccessQuestion(ctx, questions)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_display_result", "ctx", undefined), new $runtime.NonTerminal("repeat_command_phrase", undefined, undefined)], $locale._("${ctx} ${repeat_command_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.repeatCommandReply(ctx)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_action_success", "ctx", undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${stmt}"), ((ctx : S.ContextInfo, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_action_success", "ctx", undefined), new $runtime.NonTerminal("yes_please_phrase", undefined, undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${yes_please_phrase} ${stmt}"), ((ctx : S.ContextInfo, $0 : string, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_display_result", "ctx", undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${stmt}"), ((ctx : S.ContextInfo, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_display_result", "ctx", undefined), new $runtime.NonTerminal("yes_please_phrase", undefined, undefined), new $runtime.NonTerminal("initial_command", "stmt", undefined)], $locale._("${ctx} ${yes_please_phrase} ${stmt}"), ((ctx : S.ContextInfo, $0 : string, stmt : Ast.Expression) : (Ast.DialogueState)|null => D.startNewRequest($loader, ctx, stmt)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_action_error", "ctx", undefined), new $runtime.NonTerminal("no_thanks_phrase", undefined, undefined)], $locale._("${ctx} ${no_thanks_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'cancel', null)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_action_error", "ctx", undefined), new $runtime.NonTerminal("nevermind_phrase", undefined, undefined)], $locale._("${ctx} ${nevermind_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'cancel', null)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_action_error", "ctx", undefined), new $runtime.NonTerminal("imprecise_slot_fill_answer_phrase", "answer", undefined)], $locale._("${ctx} {i see ,|okay|okay ,|yeah|yeah ,} {can you try|how about} ${answer} ?"), ((ctx : S.ContextInfo, answer : Ast.Value|C.InputParamSlot) : (Ast.DialogueState)|null => D.actionErrorChangeParam(ctx, answer)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_empty_search", "ctx", undefined), new $runtime.NonTerminal("precise_empty_search_change_request", "req", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "currentTableFunction" as ("currentTableFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${req}"), ((ctx : S.ContextInfo, req : Ast.Expression) : (Ast.DialogueState)|null => D.preciseEmptySearchChangeRequest(ctx, req)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_empty_search", "ctx", undefined), new $runtime.NonTerminal("imprecise_empty_search_change_request", "req", undefined)], $locale._("${ctx} ${req}"), ((ctx : S.ContextInfo, req : Ast.Value|C.FilterSlot) : (Ast.DialogueState)|null => D.impreciseEmptySearchChangeRequest(ctx, req)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_empty_search", "ctx", undefined), new $runtime.NonTerminal("accept_phrase", undefined, undefined)], $locale._("${ctx} ${accept_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.acceptEmptySearchOffer(ctx)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_search_question", "ctx", undefined), new $runtime.NonTerminal("precise_search_question_answer", "answer", ["functionName" as ("functionName" extends keyof ReturnType<typeof D.preciseSearchQuestionAnswerKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${answer}"), ((ctx : S.ContextInfo, answer : D.PreciseSearchQuestionAnswer) : (Ast.DialogueState)|null => D.preciseSearchQuestionAnswer(ctx, answer)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_search_question", "ctx", undefined), new $runtime.NonTerminal("imprecise_search_question_answer", "answer", undefined)], $locale._("${ctx} ${answer}"), ((ctx : S.ContextInfo, answer : Ast.Value|C.FilterSlot|'dontcare') : (Ast.DialogueState)|null => D.impreciseSearchQuestionAnswer(ctx, answer)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_slot_fill", "ctx", undefined), new $runtime.NonTerminal("precise_slot_fill_answer", "answer", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.invocationKeyFn> ? string : void), 0, "nextFunction" as ("nextFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${answer}"), ((ctx : S.ContextInfo, answer : Ast.Invocation) : (Ast.DialogueState)|null => D.preciseSlotFillAnswer(ctx, answer)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_slot_fill", "ctx", undefined), new $runtime.NonTerminal("imprecise_slot_fill_answer", "answer", undefined)], $locale._("${ctx} ${answer}"), ((ctx : S.ContextInfo, answer : Ast.Value|C.InputParamSlot) : (Ast.DialogueState)|null => D.impreciseSlotFillAnswer(ctx, answer)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", undefined), new $runtime.NonTerminal("ask_recommend_phrase", undefined, undefined)], $locale._("${ctx} ${ask_recommend_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'ask_recommend', null)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_search_question", "ctx", undefined), new $runtime.NonTerminal("ask_recommend_phrase", undefined, undefined)], $locale._("${ctx} ${ask_recommend_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => S.makeSimpleState(ctx, 'ask_recommend', null)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_propose_refined_query", "ctx", undefined), new $runtime.NonTerminal("accept_phrase", undefined, undefined)], $locale._("${ctx} ${accept_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.positiveProposalReply(ctx)), undefined, { weight: (0.5) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_propose_refined_query", "ctx", undefined), new $runtime.NonTerminal("negative_proposal_reply", "reply", ["functionName" as ("functionName" extends keyof ReturnType<typeof D.negativeProposalReplyKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${reply}"), ((ctx : S.ContextInfo, reply : D.NegativeProposalReply) : (Ast.DialogueState)|null => D.negativeProposalReply(ctx, reply)), undefined, { weight: (0.5) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("accept_phrase", undefined, undefined)], $locale._("${ctx} ${accept_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.positiveRecommendationReply($loader, ctx, null, null)), undefined, { weight: (0.7) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("recommendation_accept_phrase_with_action", "acceptedAction", undefined)], $locale._("${ctx} ${acceptedAction}"), ((ctx : S.ContextInfo, acceptedAction : Ast.Invocation) : (Ast.DialogueState)|null => D.positiveRecommendationReply($loader, ctx, acceptedAction, null)), undefined, { weight: (0.7) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("recommendation_accept_phrase_by_name", "name", undefined)], $locale._("${ctx} ${name}"), ((ctx : S.ContextInfo, name : [Ast.EntityValue|Ast.NumberValue, Ast.Invocation|null]) : (Ast.DialogueState)|null => D.positiveRecommendationReply($loader, ctx, name[1], name[0])), undefined, { weight: (0.7) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("tell_me_more_phrase", undefined, undefined)], $locale._("${ctx} ${tell_me_more_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.recommendationLearnMoreReply(ctx, null)), undefined, { weight: (0.5) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("accept_phrase", undefined, undefined), new $runtime.NonTerminal("tell_me_more_phrase", undefined, undefined)], $locale._("${ctx} ${accept_phrase} ${tell_me_more_phrase}"), ((ctx : S.ContextInfo, $0 : string, $1 : string) : (Ast.DialogueState)|null => D.recommendationLearnMoreReply(ctx, null)), undefined, { weight: (0.5) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("list_proposal_tell_me_more", "name", undefined)], $locale._("${ctx} ${name}"), ((ctx : S.ContextInfo, name : Ast.EntityValue|Ast.NumberValue) : (Ast.DialogueState)|null => D.recommendationLearnMoreReply(ctx, name)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("negative_proposal_reply", "reply", ["functionName" as ("functionName" extends keyof ReturnType<typeof D.negativeProposalReplyKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${reply}"), ((ctx : S.ContextInfo, reply : D.NegativeProposalReply) : (Ast.DialogueState)|null => D.negativeRecommendationReply(ctx, reply)), undefined, { weight: (0.6) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("thanks_phrase", undefined, undefined)], $locale._("${ctx} ${thanks_phrase} {goodbye !|}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.recommendationCancelReply(ctx, true)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("ok_thanks_phrase", undefined, undefined)], $locale._("${ctx} ${ok_thanks_phrase} {goodbye !|}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.recommendationCancelReply(ctx, !ctx.aux.action && !ctx.aux.hasLearnMore)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("no_thanks_phrase", undefined, undefined)], $locale._("${ctx} ${no_thanks_phrase} {goodbye !|}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.recommendationCancelReply(ctx, ctx.aux.action || ctx.aux.hasLearnMore)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("related_question", "question", undefined)], $locale._("${ctx} ${question}"), ((ctx : S.ContextInfo, question : Ast.DialogueState) : (Ast.DialogueState)|null => question), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("user_search_question", "questions", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${questions}"), ((ctx : S.ContextInfo, questions : C.ParamSlot[]) : (Ast.DialogueState)|null => D.recommendationSearchQuestionReply(ctx, questions)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_learn_more_what", "ctx", undefined), new $runtime.NonTerminal("user_search_question", "questions", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${questions}"), ((ctx : S.ContextInfo, questions : C.ParamSlot[]) : (Ast.DialogueState)|null => D.learnMoreSearchQuestionReply(ctx, questions)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_display_result", "ctx", undefined), new $runtime.NonTerminal("user_search_question", "questions", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${questions}"), ((ctx : S.ContextInfo, questions : C.ParamSlot[]) : (Ast.DialogueState)|null => D.displayResultSearchQuestionReply(ctx, questions)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", undefined), new $runtime.NonTerminal("positive_list_proposal_reply", "reply", undefined)], $locale._("${ctx} ${reply}"), ((ctx : S.ContextInfo, reply : [Ast.EntityValue|Ast.NumberValue, Ast.Invocation|null, boolean]) : (Ast.DialogueState)|null => D.positiveListProposalReply($loader, ctx, reply)), undefined, { weight: (10 * 0.7) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", undefined), new $runtime.NonTerminal("list_proposal_accept_phrase_with_action_by_name", "action", undefined)], $locale._("${ctx} ${action}"), ((ctx : S.ContextInfo, action : Ast.Invocation) : (Ast.DialogueState)|null => D.positiveListProposalReplyActionByName($loader, ctx, action)), undefined, { weight: (10 * 0.7) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", undefined), new $runtime.NonTerminal("list_proposal_tell_me_more", "name", undefined)], $locale._("${ctx} ${name}"), ((ctx : S.ContextInfo, name : Ast.EntityValue|Ast.NumberValue) : (Ast.DialogueState)|null => D.listProposalLearnMoreReply(ctx, name)), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", undefined), new $runtime.NonTerminal("negative_proposal_reply", "reply", ["functionName" as ("functionName" extends keyof ReturnType<typeof D.negativeProposalReplyKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${reply}"), ((ctx : S.ContextInfo, reply : D.NegativeProposalReply) : (Ast.DialogueState)|null => D.negativeListProposalReply(ctx, reply)), undefined, { weight: (0.6) });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", undefined), new $runtime.NonTerminal("related_question", "question", undefined)], $locale._("${ctx} ${question}"), ((ctx : S.ContextInfo, question : Ast.DialogueState) : (Ast.DialogueState)|null => question), undefined, {  });
   $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", undefined), new $runtime.NonTerminal("user_search_question_for_list", "questions", ["functionName" as ("functionName" extends keyof ReturnType<typeof D.userSearchQuestionForListKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${ctx} ${questions}"), ((ctx : S.ContextInfo, questions : D.UserSearchQuestionForList) : (Ast.DialogueState)|null => D.listProposalSearchQuestionReply(ctx, questions)), undefined, {  });
   if (!$options.flags.nostream) {
       $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_action_success", "ctx", undefined), new $runtime.NonTerminal("add_separate_stream", "stream", undefined)], $locale._("${ctx} ${stream}"), ((ctx : S.ContextInfo, stream : Ast.Expression) : (Ast.DialogueState)|null => D.addStream(ctx, stream)), undefined, {  });
       $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", undefined), new $runtime.NonTerminal("add_separate_stream_to_query", "stream", undefined)], $locale._("${ctx} ${stream}"), ((ctx : S.ContextInfo, stream : Ast.Expression) : (Ast.DialogueState)|null => D.addStream(ctx, stream)), undefined, {  });
       $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", undefined), new $runtime.NonTerminal("add_separate_stream_to_query", "stream", undefined)], $locale._("${ctx} ${stream}"), ((ctx : S.ContextInfo, stream : Ast.Expression) : (Ast.DialogueState)|null => D.addStream(ctx, stream)), undefined, {  });
       $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_display_result", "ctx", undefined), new $runtime.NonTerminal("add_separate_stream_to_query", "stream", undefined)], $locale._("${ctx} ${stream}"), ((ctx : S.ContextInfo, stream : Ast.Expression) : (Ast.DialogueState)|null => D.addStream(ctx, stream)), undefined, {  });
       $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_one", "ctx", ["is_monitorable" as ("is_monitorable" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void), (true) as ReturnType<typeof S.contextKeyFn>["is_monitorable"]]), new $runtime.NonTerminal("make_monitor_phrase", undefined, undefined)], $locale._("${ctx} ${make_monitor_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.makeMonitor(ctx)), undefined, {  });
       $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_recommend_many", "ctx", ["is_monitorable" as ("is_monitorable" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void), (true) as ReturnType<typeof S.contextKeyFn>["is_monitorable"]]), new $runtime.NonTerminal("make_monitor_phrase", undefined, undefined)], $locale._("${ctx} ${make_monitor_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.makeMonitor(ctx)), undefined, {  });
       $grammar.addRule("$user", [new $runtime.NonTerminal("ctx_sys_display_result", "ctx", ["is_monitorable" as ("is_monitorable" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void), (true) as ReturnType<typeof S.contextKeyFn>["is_monitorable"]]), new $runtime.NonTerminal("make_monitor_phrase", undefined, undefined)], $locale._("${ctx} ${make_monitor_phrase}"), ((ctx : S.ContextInfo, $0 : string) : (Ast.DialogueState)|null => D.makeMonitor(ctx)), undefined, {  });
   }
};
