// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

// Templates that describe results from a query

import assert from 'assert';
import { Ast, } from 'thingtalk';
import * as C from '../../ast_manip';
import * as D from '../../dialogue_acts';
import * as S from '../../state_manip';
import { SlotBag } from '../../slot_bag';
import ThingpediaLoader from '../../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {
   $grammar.declareSymbol("direct_answer_phrase");
   $grammar.addRule("direct_answer_phrase", [new $runtime.NonTerminal("ctx_result_name", "name", undefined), new $runtime.NonTerminal("result_info_phrase", "info", undefined)], $locale._("${name} is a ${info[plural=one]}"), ((name : S.ContextName, info : D.ResultPhrase) : (D.DirectAnswerPhrase)|null => D.checkDirectAnswerPhrase(info, name.name)), D.directAnswerPhraseKeyFn, {  });
   $grammar.addRule("direct_answer_phrase", [new $runtime.NonTerminal("ctx_result_name", "name", undefined), new $runtime.NonTerminal("avp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextNameKeyFn> ? string : void)])], $locale._("${name} ${filter[plural=one]}"), ((name : S.ContextName, filter : C.FilterSlot) : (D.DirectAnswerPhrase)|null => D.makeFilterStyleDirectAnswerPhrase(name.ctx, name.name, filter)), D.directAnswerPhraseKeyFn, { priority: (0.1) });
   $grammar.addRule("direct_answer_phrase", [new $runtime.NonTerminal("ctx_result_name", "name", undefined), new $runtime.NonTerminal("pvp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextNameKeyFn> ? string : void)])], $locale._("${name} is ${filter[plural=one]}"), ((name : S.ContextName, filter : C.FilterSlot) : (D.DirectAnswerPhrase)|null => D.makeFilterStyleDirectAnswerPhrase(name.ctx, name.name, filter)), D.directAnswerPhraseKeyFn, { priority: (0.1) });
   $grammar.addRule("direct_answer_phrase", [new $runtime.NonTerminal("ctx_result_name", "name", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextNameKeyFn> ? string : void)])], $locale._("${name} is ${filter[plural=one]}"), ((name : S.ContextName, filter : C.FilterSlot) : (D.DirectAnswerPhrase)|null => D.makeFilterStyleDirectAnswerPhrase(name.ctx, name.name, filter)), D.directAnswerPhraseKeyFn, { priority: (0.1) });
   $grammar.addRule("direct_answer_phrase", [new $runtime.NonTerminal("ctx_result_name", "name", undefined), new $runtime.NonTerminal("npp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextNameKeyFn> ? string : void)])], $locale._("${name} has ${filter[plural=one]}"), ((name : S.ContextName, filter : C.FilterSlot) : (D.DirectAnswerPhrase)|null => D.makeFilterStyleDirectAnswerPhrase(name.ctx, name.name, filter)), D.directAnswerPhraseKeyFn, { priority: (0.1) });
   $grammar.declareSymbol("direct_answer_phrase_0");
   $grammar.addRule("direct_answer_phrase_0", [new $runtime.NonTerminal("direct_answer_phrase", undefined, ["index" as ("index" extends keyof ReturnType<typeof D.directAnswerPhraseKeyFn> ? string : void), (0) as ReturnType<typeof D.directAnswerPhraseKeyFn>["index"]])], "${direct_answer_phrase}", (($0 : D.DirectAnswerPhrase) : (D.DirectAnswerPhrase)|null => { return $0; }), D.directAnswerPhraseKeyFn, { identity: (true) });
   $grammar.declareSymbol("direct_answer_phrase_1");
   $grammar.addRule("direct_answer_phrase_1", [new $runtime.NonTerminal("direct_answer_phrase", undefined, ["index" as ("index" extends keyof ReturnType<typeof D.directAnswerPhraseKeyFn> ? string : void), (1) as ReturnType<typeof D.directAnswerPhraseKeyFn>["index"]])], "${direct_answer_phrase}", (($0 : D.DirectAnswerPhrase) : (D.DirectAnswerPhrase)|null => { return $0; }), D.directAnswerPhraseKeyFn, { identity: (true) });
   $grammar.declareSymbol("direct_answer_phrase_2");
   $grammar.addRule("direct_answer_phrase_2", [new $runtime.NonTerminal("direct_answer_phrase", undefined, ["index" as ("index" extends keyof ReturnType<typeof D.directAnswerPhraseKeyFn> ? string : void), (2) as ReturnType<typeof D.directAnswerPhraseKeyFn>["index"]])], "${direct_answer_phrase}", (($0 : D.DirectAnswerPhrase) : (D.DirectAnswerPhrase)|null => { return $0; }), D.directAnswerPhraseKeyFn, { identity: (true) });
   $grammar.declareSymbol("direct_answer_phrase_3");
   $grammar.addRule("direct_answer_phrase_3", [new $runtime.NonTerminal("direct_answer_phrase", undefined, ["index" as ("index" extends keyof ReturnType<typeof D.directAnswerPhraseKeyFn> ? string : void), (3) as ReturnType<typeof D.directAnswerPhraseKeyFn>["index"]])], "${direct_answer_phrase}", (($0 : D.DirectAnswerPhrase) : (D.DirectAnswerPhrase)|null => { return $0; }), D.directAnswerPhraseKeyFn, { identity: (true) });
   $grammar.declareSymbol("direct_result_info_phrase");
   $grammar.addRule("direct_result_info_phrase", [new $runtime.NonTerminal("one_param_direct_result_info_phrase", undefined, undefined)], "${one_param_direct_result_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { identity: (true) });
   $grammar.addRule("direct_result_info_phrase", [new $runtime.NonTerminal("two_param_direct_result_info_phrase", undefined, undefined)], "${two_param_direct_result_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { identity: (true) });
   $grammar.addRule("direct_result_info_phrase", [new $runtime.NonTerminal("three_param_direct_result_info_phrase", undefined, undefined)], "${three_param_direct_result_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { identity: (true) });
   $grammar.declareSymbol("one_param_direct_result_info_phrase");
   $grammar.addRule("one_param_direct_result_info_phrase", [new $runtime.NonTerminal("ctx_with_result", "ctx", undefined), new $runtime.NonTerminal("out_param_Any", "param", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)]), new $runtime.NonTerminal("constant_Any", "v", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("the ${param} is ${v}"), ((ctx : S.ContextInfo, param : C.ParamSlot, v : Ast.Value) : (D.ResultPhrase)|null => {
        assert(param.type.equals(v.getType()));
        const bag = new SlotBag(param.schema);
        bag.set(param.name, v);
        return D.makeResultPhrase(ctx, bag);
    }), D.resultPhraseKeyFn, {  });
   $grammar.addRule("one_param_direct_result_info_phrase", [new $runtime.NonTerminal("ctx_with_result", "ctx", undefined), new $runtime.NonTerminal("constant_Measure_m", "v", undefined)], $locale._("the distance is ${v}"), ((ctx : S.ContextInfo, v : Ast.Value) : (D.ResultPhrase)|null => {
        const bag = new SlotBag(null);
        bag.set('distance', v);
        return D.makeResultPhrase(ctx, bag);
    }), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("two_param_direct_result_info_phrase");
   $grammar.addRule("two_param_direct_result_info_phrase", [new $runtime.NonTerminal("one_param_direct_result_info_phrase", "b1", undefined), new $runtime.NonTerminal("one_param_direct_result_info_phrase", "b2", undefined)], $locale._("${b1} and ${b2}"), ((b1 : D.ResultPhrase, b2 : D.ResultPhrase) : (D.ResultPhrase)|null => D.mergeResultPhrase(b1, b2)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("three_param_direct_result_info_phrase");
   $grammar.addRule("three_param_direct_result_info_phrase", [new $runtime.NonTerminal("one_param_direct_result_info_phrase", "b1", undefined), new $runtime.NonTerminal("two_param_direct_result_info_phrase", "b2", undefined)], $locale._("${b1}, ${b2}"), ((b1 : D.ResultPhrase, b2 : D.ResultPhrase) : (D.ResultPhrase)|null => D.mergeResultPhrase(b1, b2)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("result_info_phrase");
   $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("one_with_info_phrase", undefined, undefined)], "${one_with_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33*2), priority: (1), identity: (true) });
   $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("one_which_info_phrase", undefined, undefined)], "${one_which_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33*2), priority: (1), identity: (true) });
   $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", undefined, undefined)], "${one_clean_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33*2), priority: (1), identity: (true) });
   $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("two_with_info_phrase", undefined, undefined)], "${two_with_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33), priority: (2), identity: (true) });
   $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("two_which_info_phrase", undefined, undefined)], "${two_which_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33), priority: (2), identity: (true) });
   $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", undefined, undefined)], "${two_clean_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33), priority: (2), identity: (true) });
   if (!$options.flags.inference) {
       $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("three_with_info_phrase", undefined, undefined)], "${three_with_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33*0.5), identity: (true) });
       $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("three_which_info_phrase", undefined, undefined)], "${three_which_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33*0.5), identity: (true) });
       $grammar.addRule("result_info_phrase", [new $runtime.NonTerminal("three_clean_info_phrase", undefined, undefined)], "${three_clean_info_phrase}", (($0 : D.ResultPhrase) : (D.ResultPhrase)|null => { return $0; }), D.resultPhraseKeyFn, { weight: (0.33*0.5), identity: (true) });
   }
   $grammar.declareSymbol("base_result_phrase");
   $grammar.addRule("base_result_phrase", [new $runtime.NonTerminal("ctx_with_result", "ctx", undefined), new $runtime.NonTerminal("base_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "currentFunction" as ("currentFunction" extends keyof ReturnType<typeof S.contextKeyFn> ? string : void)])], $locale._("${table}"), ((ctx : S.ContextInfo, table : Ast.InvocationExpression) : (D.ResultPhrase)|null => {
        assert(table.schema instanceof Ast.FunctionDef);
        return D.makeResultPhrase(ctx, new SlotBag(table.schema));
    }), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("one_with_info_phrase");
   $grammar.addRule("one_with_info_phrase", [new $runtime.NonTerminal("base_result_phrase", "table", undefined), new $runtime.NonTerminal("npp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {with|having} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("one_which_info_phrase");
   $grammar.addRule("one_which_info_phrase", [new $runtime.NonTerminal("base_result_phrase", "table", undefined), new $runtime.NonTerminal("npi_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("one_which_info_phrase", [new $runtime.NonTerminal("base_result_phrase", "table", undefined), new $runtime.NonTerminal("avp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{${filter[plural=one]}}other{${filter[plural=other]}}} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("one_which_info_phrase", [new $runtime.NonTerminal("base_result_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, { priority: (-0.1) });
   $grammar.addRule("one_which_info_phrase", [new $runtime.NonTerminal("base_result_phrase", "table", undefined), new $runtime.NonTerminal("npv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("one_clean_info_phrase");
   $grammar.addRule("one_clean_info_phrase", [new $runtime.NonTerminal("base_result_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${filter} ${table} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("one_clean_info_phrase", [new $runtime.NonTerminal("base_result_phrase", "table", undefined), new $runtime.NonTerminal("pvp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("one_clean_info_phrase", [new $runtime.NonTerminal("base_result_phrase", "table", undefined), new $runtime.NonTerminal("preposition_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("two_with_info_phrase");
   $grammar.addRule("two_with_info_phrase", [new $runtime.NonTerminal("one_with_info_phrase", "table", undefined), new $runtime.NonTerminal("npp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_with_info_phrase", [new $runtime.NonTerminal("one_which_info_phrase", "table", undefined), new $runtime.NonTerminal("npp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {and having|and with|, with} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_with_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("npp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {with|having} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("two_which_info_phrase");
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_with_info_phrase", "table", undefined), new $runtime.NonTerminal("npi_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_with_info_phrase", "table", undefined), new $runtime.NonTerminal("avp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{${filter[plural=one]}}other{${filter[plural=other]}}} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_with_info_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, { priority: (-0.1) });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_with_info_phrase", "table", undefined), new $runtime.NonTerminal("npv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_which_info_phrase", "table", undefined), new $runtime.NonTerminal("npi_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_which_info_phrase", "table", undefined), new $runtime.NonTerminal("avp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${table[plural]:select:one{${filter[plural=one]}}other{${filter[plural=other]}}} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_which_info_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, { priority: (-0.1) });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_which_info_phrase", "table", undefined), new $runtime.NonTerminal("npv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("npi_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("avp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{${filter[plural=one]}}other{${filter[plural=other]}}} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, { priority: (-0.1) });
   $grammar.addRule("two_which_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("npv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("two_clean_info_phrase");
   $grammar.addRule("two_clean_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${filter} ${table} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_clean_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("pvp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("two_clean_info_phrase", [new $runtime.NonTerminal("one_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("preposition_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("three_with_info_phrase");
   $grammar.addRule("three_with_info_phrase", [new $runtime.NonTerminal("two_with_info_phrase", "table", undefined), new $runtime.NonTerminal("npp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_with_info_phrase", [new $runtime.NonTerminal("two_which_info_phrase", "table", undefined), new $runtime.NonTerminal("npp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {and having|and with|, with} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_with_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("npp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {with|having} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("three_which_info_phrase");
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_with_info_phrase", "table", undefined), new $runtime.NonTerminal("npi_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_with_info_phrase", "table", undefined), new $runtime.NonTerminal("avp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{${filter[plural=one]}}other{${filter[plural=other]}}} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_with_info_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, { priority: (-0.1) });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_with_info_phrase", "table", undefined), new $runtime.NonTerminal("npv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_which_info_phrase", "table", undefined), new $runtime.NonTerminal("npi_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_which_info_phrase", "table", undefined), new $runtime.NonTerminal("avp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${table[plural]:select:one{${filter[plural=one]}}other{${filter[plural=other]}}} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_which_info_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, { priority: (-0.1) });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_which_info_phrase", "table", undefined), new $runtime.NonTerminal("npv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} and ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("npi_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("avp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{${filter[plural=one]}}other{${filter[plural=other]}}} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, { priority: (-0.1) });
   $grammar.addRule("three_which_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("npv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} {which|that} ${table[plural]:select:one{is}other{are}} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.declareSymbol("three_clean_info_phrase");
   $grammar.addRule("three_clean_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("apv_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${filter} ${table} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_clean_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("pvp_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
   $grammar.addRule("three_clean_info_phrase", [new $runtime.NonTerminal("two_clean_info_phrase", "table", undefined), new $runtime.NonTerminal("preposition_filter", "filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof D.resultPhraseKeyFn> ? string : void)])], $locale._("${table} ${filter} [plural=table[plural]]"), ((table : D.ResultPhrase, filter : C.FilterSlot) : (D.ResultPhrase)|null => D.addSlotToResultPhrase(table, filter)), D.resultPhraseKeyFn, {  });
};
