// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

// Templates for converting one-shot commands into continuous commands

import { Ast, } from 'thingtalk';
import * as C from '../../ast_manip';
import * as S from '../../state_manip';
import ThingpediaLoader from '../../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {
   $grammar.declareSymbol("notification_preamble");
   $grammar.addRule("notification_preamble", [new $runtime.NonTerminal("ctx_with_notification", "ctx", undefined), new $runtime.NonTerminal("ctx_notification_app_name", "app", undefined)], $locale._("notification from ${app}:"), ((ctx : S.ContextInfo, app : Ast.Value) : (S.ContextInfo)|null => ctx), S.contextKeyFn, {  });
   $grammar.addRule("notification_preamble", [new $runtime.NonTerminal("ctx_with_notification", "ctx", undefined)], $locale._(""), ((ctx : S.ContextInfo) : (S.ContextInfo)|null => ctx.state.dialogueActParam === null ? ctx : null), S.contextKeyFn, { priority: (-1) });
   $grammar.declareSymbol("notification_error_preamble");
   $grammar.addRule("notification_error_preamble", [new $runtime.NonTerminal("sorry_preamble", "sorry", undefined), new $runtime.NonTerminal("ctx_with_notification", "ctx", undefined), new $runtime.NonTerminal("ctx_notification_app_name", "app", undefined)], $locale._("${sorry} {there was an error in|an error occurred in} ${app}:"), ((sorry : string, ctx : S.ContextInfo, app : Ast.Value) : (S.ContextInfo)|null => ctx), S.contextKeyFn, {  });
   $grammar.addRule("notification_error_preamble", [new $runtime.NonTerminal("sorry_preamble", "sorry", undefined), new $runtime.NonTerminal("ctx_with_notification", "ctx", undefined)], $locale._("${sorry} {there was an error}:"), ((sorry : string, ctx : S.ContextInfo) : (S.ContextInfo)|null => ctx.state.dialogueActParam === null ? ctx : null), S.contextKeyFn, {  });
   $grammar.declareSymbol("add_separate_stream_to_query");
   if ($options.flags.timer) {
       $grammar.addRule("add_separate_stream_to_query", [new $runtime.NonTerminal("timer", "stream", undefined)], $locale._("{send this to me|send it to me|i want to see this|i want this|send this} ${stream}"), ((stream : Ast.FunctionCallExpression) : (Ast.Expression)|null => stream), C.expressionKeyFn, {  });
   }
   $grammar.addRule("add_separate_stream_to_query", [new $runtime.NonTerminal("stream", "stream", undefined)], $locale._("{send this to me|send it to me|i want to see this|i want this|send this} ${stream}"), ((stream : Ast.Expression) : (Ast.Expression)|null => stream), C.expressionKeyFn, {  });
   $grammar.declareSymbol("add_separate_stream");
   $grammar.addRule("add_separate_stream", [new $runtime.NonTerminal("stream", "stream", undefined)], $locale._("{do this|do it|do it automatically} ${stream}"), ((stream : Ast.Expression) : (Ast.Expression)|null => stream), C.expressionKeyFn, {  });
   if ($options.flags.timer) {
       $grammar.addRule("add_separate_stream", [new $runtime.NonTerminal("timer", "stream", undefined)], $locale._("{do this|do it|do it automatically} ${stream}"), ((stream : Ast.FunctionCallExpression) : (Ast.Expression)|null => stream), C.expressionKeyFn, {  });
   }
};
