// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2019-2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>
//         Silei Xu <silei@cs.stanford.edu>

import assert from 'assert';
import { Ast, Type } from 'thingtalk';
import * as C from '../ast_manip';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {
   $grammar.declareSymbol("table_join_replace_placeholder");
   $grammar.declareSymbol("action_replace_param_with_table");
   $grammar.declareSymbol("action_replace_param_with_stream");
   $grammar.declareSymbol("the_out_param_Any");
   $grammar.addRule("the_out_param_Any", [new $runtime.NonTerminal("out_param_Any", "p", undefined)], $locale._("the ${p}"), ((p : C.ParamSlot) : (C.ParamSlot)|null => p), C.paramKeyFn, {  });
   if (!$options.flags.inference) {
       if (!$options.flags.turking) {
           $grammar.addRule("the_out_param_Any", [new $runtime.NonTerminal("out_param_Any", "p", undefined)], $locale._("its ${p}"), ((p : C.ParamSlot) : (C.ParamSlot)|null => p), C.paramKeyFn, {  });
           $grammar.addRule("the_out_param_Any", [new $runtime.NonTerminal("out_param_Any", "p", undefined)], $locale._("their ${p}"), ((p : C.ParamSlot) : (C.ParamSlot)|null => p), C.paramKeyFn, {  });
       }
   }
   $grammar.declareSymbol("the_out_param_pair");
   $grammar.addRule("the_out_param_pair", [new $runtime.NonTerminal("out_param_Any", "p1", undefined), new $runtime.NonTerminal("out_param_Any", "p2", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("the ${p1} and ${p2}"), ((p1 : C.ParamSlot, p2 : C.ParamSlot) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(p1.schema, p2.schema));
        if (p1.name === p2.name)
            return null;
        return [p1, p2];
    }), C.paramArrayKeyFn, {  });
   $grammar.declareSymbol("the_out_param_triple");
   $grammar.addRule("the_out_param_triple", [new $runtime.NonTerminal("out_param_Any", "p1", undefined), new $runtime.NonTerminal("out_param_Any", "p2", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), new $runtime.NonTerminal("out_param_Any", "p3", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("the ${p1} , ${p2} {,|} and ${p3}"), ((p1 : C.ParamSlot, p2 : C.ParamSlot, p3 : C.ParamSlot) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(p1.schema, p2.schema));
        assert(C.isSameFunction(p1.schema, p3.schema));
        if (p1.name === p2.name || p1.name === p3.name || p2.name === p3.name)
            return null;
        return [p1, p2, p3];
    }), C.paramArrayKeyFn, {  });
   $grammar.declareSymbol("pair_projection");
   $grammar.addRule("pair_projection", [new $runtime.NonTerminal("the_out_param_pair", "outParams", undefined), new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)])], $locale._("{both|} ${outParams} of ${table}"), ((outParams : C.ParamSlot[], table : Ast.Expression) : (Ast.Expression)|null => C.makeMultiFieldProjection($loader, 'table', table, outParams)), C.expressionKeyFn, {  });
   $grammar.addRule("pair_projection", [new $runtime.NonTerminal("the_out_param_triple", "outParams", undefined), new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)])], $locale._("${outParams} of the ${table}"), ((outParams : C.ParamSlot[], table : Ast.Expression) : (Ast.Expression)|null => C.makeMultiFieldProjection($loader, 'table', table, outParams)), C.expressionKeyFn, {  });
   $grammar.declareSymbol("where_question");
   $grammar.addRule("where_question", [new $runtime.NonTerminal("with_filtered_table", "table", ["has_geo" as ("has_geo" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["has_geo"]])], $locale._("where is the ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Location, table, 'geo')), C.expressionKeyFn, {  });
   $grammar.addRule("where_question", [new $runtime.NonTerminal("with_arg_min_max_table", "table", ["has_geo" as ("has_geo" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["has_geo"]])], $locale._("where is the ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Location, table, 'geo')), C.expressionKeyFn, {  });
   $grammar.declareSymbol("boolean_projection");
   $grammar.addRule("boolean_projection", [new $runtime.NonTerminal("with_filtered_table", "table", undefined), new $runtime.NonTerminal("npp_boolean_projection", "p", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)])], $locale._("does {the||any} ${table} have ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Boolean, table, p)), C.expressionKeyFn, {  });
   $grammar.addRule("boolean_projection", [new $runtime.NonTerminal("with_filtered_table", "table", undefined), new $runtime.NonTerminal("avp_boolean_projection", "p", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)])], $locale._("does {the||any} ${table} ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Boolean, table, p)), C.expressionKeyFn, {  });
   $grammar.addRule("boolean_projection", [new $runtime.NonTerminal("with_filtered_table", "table", undefined), new $runtime.NonTerminal("pvp_boolean_projection", "p", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)])], $locale._("is {the|} ${table} ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Boolean, table, p)), C.expressionKeyFn, {  });
   $grammar.addRule("boolean_projection", [new $runtime.NonTerminal("with_filtered_table", "table", undefined), new $runtime.NonTerminal("preposition_boolean_projection", "p", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)])], $locale._("is {the|} ${table} ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Boolean, table, p)), C.expressionKeyFn, {  });
   $grammar.addRule("boolean_projection", [new $runtime.NonTerminal("with_filtered_table", "table", undefined), new $runtime.NonTerminal("npi_boolean_projection", "p", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)])], $locale._("is {the|} ${table} ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Boolean, table, p)), C.expressionKeyFn, {  });
   $grammar.addRule("boolean_projection", [new $runtime.NonTerminal("with_filtered_table", "table", undefined), new $runtime.NonTerminal("npv_boolean_projection", "p", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)])], $locale._("is {the|} ${table} ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Boolean, table, p)), C.expressionKeyFn, {  });
   $grammar.addRule("boolean_projection", [new $runtime.NonTerminal("with_filtered_table", "table", undefined), new $runtime.NonTerminal("apv_boolean_projection", "p", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)])], $locale._("is {the|} ${table} ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', Type.Boolean, table, p)), C.expressionKeyFn, {  });
   for (const proj of $loader.projections) {
       if (proj.category === 'pvp' || proj.category === 'preposition') {
           $grammar.declareSymbol("question_projection_table");
           $grammar.addRule("question_projection_table", [new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.pronoun) + " " + (proj.base) + " {is|was|are|were} {the|} ${table} " + (proj.canonical) + ""), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           $grammar.declareSymbol("question_projection_table");
           $grammar.addRule("question_projection_table", [new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.pronoun) + " " + (proj.base) + " {is|was|are|were} {the|} ${table} " + (proj.canonical) + ""), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           if (proj.base) {
               $grammar.declareSymbol("command_projection_table");
               $grammar.addRule("command_projection_table", [new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.base) + " {that|which|} {the|} ${table} {is|was|are|were} " + (proj.canonical) + ""), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
               $grammar.declareSymbol("command_projection_table");
               $grammar.addRule("command_projection_table", [new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.base) + " {that|which|} {the|} ${table} {is|was|are|were} " + (proj.canonical) + ""), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           }
       }
       if (proj.category === 'avp') {
           $grammar.declareSymbol("question_projection_table");
           $grammar.addRule("question_projection_table", [new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.pronoun) + " " + (proj.base) + " {do|does|did} {the|} ${table} " + (proj.canonical) + ""), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           $grammar.declareSymbol("question_projection_table");
           $grammar.addRule("question_projection_table", [new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.pronoun) + " " + (proj.base) + " {do|does|did} {the|} ${table} " + (proj.canonical) + ""), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           if (proj.base) {
               $grammar.declareSymbol("command_projection_table");
               $grammar.addRule("command_projection_table", [new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.base) + " {that|which|} {the|} ${table} " + (proj.canonical) + ""), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
               $grammar.declareSymbol("command_projection_table");
               $grammar.addRule("command_projection_table", [new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.base) + " {that|which|} {the|} ${table} " + (proj.canonical) + ""), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           }
       }
       if (proj.category === 'reverse_verb') {
           $grammar.declareSymbol("question_projection_table");
           $grammar.addRule("question_projection_table", [new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.pronoun) + " " + (proj.base) + " " + (proj.canonical) + " {the|} ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           $grammar.declareSymbol("question_projection_table");
           $grammar.addRule("question_projection_table", [new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.pronoun) + " " + (proj.base) + " " + (proj.canonical) + " {the|} ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           if (proj.base) {
               $grammar.declareSymbol("command_projection_table");
               $grammar.addRule("command_projection_table", [new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.base) + " {that|which|} " + (proj.canonical) + " {the|} ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
               $grammar.declareSymbol("command_projection_table");
               $grammar.addRule("command_projection_table", [new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]])], $locale._("" + (proj.base) + " {that|which|} " + (proj.canonical) + " {the|} ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, proj.pslot)), C.expressionKeyFn, {  });
           }
       }
   }
   $grammar.declareSymbol("projection_Any");
   $grammar.addRule("projection_Any", [new $runtime.NonTerminal("out_param_Any", "outParam", undefined), new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("the ${outParam} of ${table}"), ((outParam : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, outParam)), C.expressionKeyFn, {  });
   $grammar.addRule("projection_Any", [new $runtime.NonTerminal("out_param_Any", "outParam", undefined), new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("the ${outParam} of the ${table}"), ((outParam : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, outParam)), C.expressionKeyFn, {  });
   $grammar.declareSymbol("verb_projection_Any");
   $grammar.addRule("verb_projection_Any", [new $runtime.NonTerminal("out_param_Any", "outParam", undefined), new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("${outParam} {do|does} {the|} ${table} have"), ((outParam : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, outParam)), C.expressionKeyFn, {  });
   $grammar.addRule("verb_projection_Any", [new $runtime.NonTerminal("out_param_Any", "outParam", undefined), new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("${outParam} {do|does} {the|} ${table} have"), ((outParam : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'table', null, table, outParam)), C.expressionKeyFn, {  });
   $grammar.declareSymbol("stream_projection_Any");
   if (!$options.flags.nostream) {
       $grammar.addRule("stream_projection_Any", [new $runtime.NonTerminal("out_param_Any", "outParam", undefined), new $runtime.NonTerminal("with_filtered_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("the ${outParam} of new ${table}"), ((outParam : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'stream', null, table, outParam)), C.expressionKeyFn, {  });
       $grammar.addRule("stream_projection_Any", [new $runtime.NonTerminal("out_param_Any", "outParam", undefined), new $runtime.NonTerminal("with_arg_min_max_table", "table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], $locale._("the ${outParam} of the new ${table}"), ((outParam : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => C.makeSingleFieldProjection($loader, 'stream', null, table, outParam)), C.expressionKeyFn, {  });
       $grammar.addRule("stream_projection_Any", [new $runtime.NonTerminal("with_filtered_table", "table", ["is_monitorable" as ("is_monitorable" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_monitorable"]])], $locale._("new ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => C.makeTypeBasedStreamProjection(table)), C.expressionKeyFn, {  });
   }
   $grammar.declareSymbol("placeholder");
   $grammar.addRule("placeholder", [], "some number", (() : (C.Placeholder)|null => ({ type: Type.Number })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some value", (() : (C.Placeholder)|null => ({ type: Type.Number })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "something", (() : (C.Placeholder)|null => ({ type: Type.String })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some value", (() : (C.Placeholder)|null => ({ type: Type.Currency })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some amount", (() : (C.Placeholder)|null => ({ type: Type.Currency })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some time", (() : (C.Placeholder)|null => ({ type: Type.Time })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain time", (() : (C.Placeholder)|null => ({ type: Type.Time })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some date", (() : (C.Placeholder)|null => ({ type: Type.Date })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain date", (() : (C.Placeholder)|null => ({ type: Type.Date })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some time", (() : (C.Placeholder)|null => ({ type: Type.Date })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain time", (() : (C.Placeholder)|null => ({ type: Type.Date })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some location", (() : (C.Placeholder)|null => ({ type: Type.Location })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a location", (() : (C.Placeholder)|null => ({ type: Type.Location })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some place", (() : (C.Placeholder)|null => ({ type: Type.Location })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a place", (() : (C.Placeholder)|null => ({ type: Type.Location })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain place", (() : (C.Placeholder)|null => ({ type: Type.Location })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "somewhere", (() : (C.Placeholder)|null => ({ type: Type.Location })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some duration", (() : (C.Placeholder)|null => ({ type: new Type.Measure('ms') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some amount of time", (() : (C.Placeholder)|null => ({ type: new Type.Measure('ms') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some size", (() : (C.Placeholder)|null => ({ type: new Type.Measure('byte') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain size", (() : (C.Placeholder)|null => ({ type: new Type.Measure('byte') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain weight", (() : (C.Placeholder)|null => ({ type: new Type.Measure('kg') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain temperature", (() : (C.Placeholder)|null => ({ type: new Type.Measure('C') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some distance", (() : (C.Placeholder)|null => ({ type: new Type.Measure('m') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some length", (() : (C.Placeholder)|null => ({ type: new Type.Measure('m') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some size", (() : (C.Placeholder)|null => ({ type: new Type.Measure('m2') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain size", (() : (C.Placeholder)|null => ({ type: new Type.Measure('m2') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some volume", (() : (C.Placeholder)|null => ({ type: new Type.Measure('m3') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain volume", (() : (C.Placeholder)|null => ({ type: new Type.Measure('m3') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some speed", (() : (C.Placeholder)|null => ({ type: new Type.Measure('mps') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "a certain speed", (() : (C.Placeholder)|null => ({ type: new Type.Measure('mps') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some image", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:picture') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some picture", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:picture') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some photo", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:picture') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "somebody", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:username') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "someone", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:username') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some one", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:username') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some person", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:username') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "somebody", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "someone", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some one", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some person", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some number", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some phone number", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "somebody", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "someone", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some one", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some person", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some email address", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some file", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some directory", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some folder", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:phone_number') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some url", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:url') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some website", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:url') })), C.placeholderKeyFn, {  });
   $grammar.addRule("placeholder", [], "some link", (() : (C.Placeholder)|null => ({ type: new Type.Entity('tt:url') })), C.placeholderKeyFn, {  });
};
