// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Silei Xu <silei@cs.stanford.edu>

import { Ast, Type } from 'thingtalk';
import * as C from '../ast_manip';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {

$loader = new ThingpediaLoader($runtime, $ttUtils, $grammar, $locale, $options);
   await (await $runtime.import("../common.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);

await $loader.init();
   await (await $runtime.import("./constants.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   $grammar.declareSymbol("table");
   $grammar.addRule("table", [new $runtime.NonTerminal("thingpedia_complete_query", undefined, undefined)], "${thingpedia_complete_query}", (($0 : Ast.Expression) : (Ast.Expression)|null => { return $0; }), C.expressionKeyFn, { identity: (true) });
   $grammar.declareSymbol("filter");
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_Any", "p", undefined), new $runtime.NonTerminal("constant_Any", "v", undefined)], $locale._("whose ${p} is ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', v)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_Any", "p", undefined), new $runtime.NonTerminal("constant_Any", "v", undefined)], $locale._("whose ${p} is not ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', v, true)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_Array__Any", "p", undefined), new $runtime.NonTerminal("constant_Any", "v", undefined)], $locale._("whose ${p} is ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, 'contains', v)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_Array__Any", "p", undefined), new $runtime.NonTerminal("constant_Any", "v", undefined)], $locale._("whose ${p} is not ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, 'contains', v, true)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.NonTerminal("constant_Numeric", "v", undefined)], $locale._("whose ${p} {is samller than|at most} ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', v)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.NonTerminal("constant_Numeric", "v", undefined)], $locale._("whose ${p} {is larger than|at least} ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', v)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_avp", "p", undefined), new $runtime.NonTerminal("constant_Any", "v", undefined)], $locale._("that ${p} ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', v)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_avp", "p", undefined), new $runtime.NonTerminal("constant_Any", "v", undefined)], $locale._("that not ${p} ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', v, true)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_avp", "p", undefined), new $runtime.NonTerminal("constant_Any", "v", undefined)], $locale._("that ${p} ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, 'contains', v)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("out_param_avp", "p", undefined), new $runtime.NonTerminal("constant_Any", "v", undefined)], $locale._("that not ${p} ${v}"), ((p : C.ParamSlot, v : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, 'contains', v, true)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("constant_Number", "v", undefined), new $runtime.NonTerminal("out_param_Any", "p", undefined)], $locale._("that has ${v} ${p}"), ((v : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'count', '*', '==', v)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("constant_Number", "v", undefined), new $runtime.NonTerminal("out_param_Any", "p", undefined)], $locale._("that has {less than|at most} ${v} ${p}"), ((v : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'count', '*', '<=', v)), C.filterKeyFn, {  });
   $grammar.addRule("filter", [new $runtime.NonTerminal("constant_Number", "v", undefined), new $runtime.NonTerminal("out_param_Any", "p", undefined)], $locale._("that has {more than|at least} ${v} ${p}"), ((v : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'count', '*', '>=', v)), C.filterKeyFn, {  });
   $grammar.declareSymbol("filtered_table");
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t", undefined), new $runtime.NonTerminal("filter", "f", undefined)], $locale._("${t} ${f}"), ((t : Ast.Expression, f : C.FilterSlot) : (Ast.Expression)|null => C.addFilter(t, f)), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("filtered_table", "t", undefined), new $runtime.NonTerminal("filter", "f", undefined)], $locale._("${t} and ${f}"), ((t : Ast.Expression, f : C.FilterSlot) : (Ast.Expression)|null => C.addFilter(t, f)), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("out_param_Any", "p", undefined), new $runtime.NonTerminal("table", "t2", undefined)], $locale._("${t1} that is ${p} of ${t2}"), ((t1 : Ast.Expression, p : C.ParamSlot, t2 : Ast.Expression) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("out_param_Any", "p", undefined), new $runtime.NonTerminal("filtered_table", "t2", undefined)], $locale._("${t1} that is ${p} of ${t2}"), ((t1 : Ast.Expression, p : C.ParamSlot, t2 : Ast.Expression) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("out_param_Array__Any", "p", undefined), new $runtime.NonTerminal("table", "t2", undefined)], $locale._("${t1} that is ${p} of ${t2}"), ((t1 : Ast.Expression, p : C.ParamSlot, t2 : Ast.Expression) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("out_param_Array__Any", "p", undefined), new $runtime.NonTerminal("filtered_table", "t2", undefined)], $locale._("${t1} that is ${p} of ${t2}"), ((t1 : Ast.Expression, p : C.ParamSlot, t2 : Ast.Expression) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("out_param_Any", "p", undefined), new $runtime.NonTerminal("table", "t2", undefined)], $locale._("${t1} that is not ${p} of ${t2}"), ((t1 : Ast.Expression, p : C.ParamSlot, t2 : Ast.Expression) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name, true);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("out_param_Any", "p", undefined), new $runtime.NonTerminal("filtered_table", "t2", undefined)], $locale._("${t1} that is not ${p} of ${t2}"), ((t1 : Ast.Expression, p : C.ParamSlot, t2 : Ast.Expression) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name, true);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("out_param_Array__Any", "p", undefined), new $runtime.NonTerminal("table", "t2", undefined)], $locale._("${t1} that is not ${p} of ${t2}"), ((t1 : Ast.Expression, p : C.ParamSlot, t2 : Ast.Expression) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name, true);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("out_param_Array__Any", "p", undefined), new $runtime.NonTerminal("filtered_table", "t2", undefined)], $locale._("${t1} that is not ${p} of ${t2}"), ((t1 : Ast.Expression, p : C.ParamSlot, t2 : Ast.Expression) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name, true);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("table", "t2", undefined), new $runtime.NonTerminal("out_param_avp", "p", undefined)], $locale._("${t1} that ${t2} ${p}"), ((t1 : Ast.Expression, t2 : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("filtered_table", "t2", undefined), new $runtime.NonTerminal("out_param_avp", "p", undefined)], $locale._("${t1} that ${t2} and ${p}"), ((t1 : Ast.Expression, t2 : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("table", "t2", undefined), new $runtime.NonTerminal("out_param_avp", "p", undefined)], $locale._("${t1} that ${t2} not ${p}"), ((t1 : Ast.Expression, t2 : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name, true);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "t1", undefined), new $runtime.NonTerminal("filtered_table", "t2", undefined), new $runtime.NonTerminal("out_param_avp", "p", undefined)], $locale._("${t1} that ${t2} and not ${p}"), ((t1 : Ast.Expression, t2 : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => {
        return C.addReverseComparisonSubquery(t1, t2, p.name, true);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "table", undefined), new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]])], $locale._("${table} that has the smallest ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeArgMaxMinTable(table, p.name, 'asc')), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("filtered_table", "table", undefined), new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]])], $locale._("${table} that has the smallest ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeArgMaxMinTable(table, p.name, 'asc')), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "table", undefined), new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]])], $locale._("${table} that has the largest ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeArgMaxMinTable(table, p.name, 'desc')), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("filtered_table", "table", undefined), new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]])], $locale._("${table} that has the largest ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => C.makeArgMaxMinTable(table, p.name, 'desc')), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "table", undefined), new $runtime.NonTerminal("out_param_Array__Any", "p", undefined)], $locale._("${table} that has the least number of ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => {
        if (!table.schema!.out[p.name] || !table.schema!.out[p.name].isArray)
            return null;
        const conflictParam = C.hasConflictParam(table, p.name, 'count');
        if (conflictParam)
            return C.makeArgMaxMinTable(table, conflictParam, 'asc');
        return C.makeAggComputeArgMinMaxExpression(table, 'count', null, p, Type.Number, 'desc');
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("filtered_table", "table", undefined), new $runtime.NonTerminal("out_param_Array__Any", "p", undefined)], $locale._("${table} that has the least number of ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => {
        if (!table.schema!.out[p.name] || !table.schema!.out[p.name].isArray)
            return null;
        const conflictParam = C.hasConflictParam(table, p.name, 'count');
        if (conflictParam)
            return C.makeArgMaxMinTable(table, conflictParam, 'asc');
        return C.makeAggComputeArgMinMaxExpression(table, 'count', null, p, Type.Number, 'desc');
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "table", undefined), new $runtime.NonTerminal("out_param_Array__Any", "p", undefined)], $locale._("${table} that has the most number of ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => {
        if (!table.schema!.out[p.name] || !table.schema!.out[p.name].isArray)
            return null;
        const conflictParam = C.hasConflictParam(table, p.name, 'count');
        if (conflictParam)
            return C.makeArgMaxMinTable(table, conflictParam, 'desc');
        return C.makeAggComputeArgMinMaxExpression(table, 'count', null, p, Type.Number, 'desc');
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("filtered_table", "table", undefined), new $runtime.NonTerminal("out_param_Array__Any", "p", undefined)], $locale._("${table} that has the most number of ${p}"), ((table : Ast.Expression, p : C.ParamSlot) : (Ast.Expression)|null => {
        if (!table.schema!.out[p.name] || !table.schema!.out[p.name].isArray)
            return null;
        const conflictParam = C.hasConflictParam(table, p.name, 'count');
        if (conflictParam)
            return C.makeArgMaxMinTable(table, conflictParam, 'desc');
        return C.makeAggComputeArgMinMaxExpression(table, 'count', null, p, Type.Number, 'desc');
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("out_param_Any", "p", undefined), new $runtime.NonTerminal("table", "table", undefined)], $locale._("${p} of ${table}"), ((p : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => {
        if (p.name === 'id')
            return null;
        return C.makeSingleFieldProjection($loader, 'table', null, table, p);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("out_param_Any", "p", undefined), new $runtime.NonTerminal("filtered_table", "table", undefined)], $locale._("${p} of ${table}"), ((p : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => {
        if (p.name === 'id')
            return null;
        return C.makeSingleFieldProjection($loader, 'table', null, table, p);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("out_param_Array__Any", "p", undefined), new $runtime.NonTerminal("table", "table", undefined)], $locale._("${p} of ${table}"), ((p : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => {
        if (p.name === 'id')
            return null;
        return C.makeSingleFieldProjection($loader, 'table', null, table, p);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("out_param_Array__Any", "p", undefined), new $runtime.NonTerminal("filtered_table", "table", undefined)], $locale._("${p} of ${table}"), ((p : C.ParamSlot, table : Ast.Expression) : (Ast.Expression)|null => {
        if (p.name === 'id')
            return null;
        return C.makeSingleFieldProjection($loader, 'table', null, table, p);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("table", "table", undefined)], $locale._("number of ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => {
        if (!table.schema!.is_list)
            return null;
        const newSchema = table.schema!.filterArguments((arg) => arg.direction !== Ast.ArgDirection.OUT)
            .addArguments([new Ast.ArgumentDef(null, Ast.ArgDirection.OUT, 'count', Type.Number)]);
        newSchema.minimal_projection = [];
        if (newSchema instanceof Ast.FunctionDef)
            newSchema.annotations.minimal_projection = new Ast.Value.Array([]);
        const agg = new Ast.AggregationExpression(null, table, '*', 'count', newSchema);
        return new Ast.ProjectionExpression(null, agg, ['count'], [], [], newSchema);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("filtered_table", "table", undefined)], $locale._("number of ${table}"), ((table : Ast.Expression) : (Ast.Expression)|null => {
        if (!table.schema!.is_list)
            return null;
        const newSchema = table.schema!.filterArguments((arg) => arg.direction !== Ast.ArgDirection.OUT)
            .addArguments([new Ast.ArgumentDef(null, Ast.ArgDirection.OUT, 'count', Type.Number)]);
        newSchema.minimal_projection = [];
        if (newSchema instanceof Ast.FunctionDef)
            newSchema.annotations.minimal_projection = new Ast.Value.Array([]);
        const agg = new Ast.AggregationExpression(null, table, '*', 'count', newSchema);
        return new Ast.ProjectionExpression(null, agg, ['count'], [], [], newSchema);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.NonTerminal("table", "t", undefined)], $locale._("total ${p} of ${t}"), ((p : C.ParamSlot, t : Ast.Expression) : (Ast.Expression)|null => {
        const schema = t.schema!;
        if (!schema.out[p.name] || !schema.out[p.name].isNumeric())
            return null;
        if (!schema.is_list)
            return null;

        const newSchema = schema.filterArguments((arg) => arg.direction !== Ast.ArgDirection.OUT || arg.name === p.name);
        newSchema.minimal_projection = [];
        if (newSchema instanceof Ast.FunctionDef)
            newSchema.annotations.minimal_projection = new Ast.Value.Array([]);
        const agg = new Ast.AggregationExpression(null, t, p.name, 'sum', newSchema);
        return new Ast.ProjectionExpression(null, agg, [p.name], [], [], newSchema);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.NonTerminal("filtered_table", "t", undefined)], $locale._("total ${p} of ${t}"), ((p : C.ParamSlot, t : Ast.Expression) : (Ast.Expression)|null => {
        const schema = t.schema!;
        if (!schema.out[p.name] || !schema.out[p.name].isNumeric())
            return null;
        if (!schema.is_list)
            return null;

        const newSchema = schema.filterArguments((arg) => arg.direction !== Ast.ArgDirection.OUT || arg.name === p.name);
        newSchema.minimal_projection = [];
        if (newSchema instanceof Ast.FunctionDef)
            newSchema.annotations.minimal_projection = new Ast.Value.Array([]);
        const agg = new Ast.AggregationExpression(null, t, p.name, 'sum', newSchema);
        return new Ast.ProjectionExpression(null, agg, [p.name], [], [], newSchema);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.NonTerminal("table", "t", undefined)], $locale._("average ${p} of ${t}"), ((p : C.ParamSlot, t : Ast.Expression) : (Ast.Expression)|null => {
        const schema = t.schema!;
        if (!schema.out[p.name] || !schema.out[p.name].isNumeric())
            return null;
        if (!schema.is_list)
            return null;

        const newSchema = schema.filterArguments((arg) => arg.direction !== Ast.ArgDirection.OUT || arg.name === p.name);
        newSchema.minimal_projection = [];
        if (newSchema instanceof Ast.FunctionDef)
            newSchema.annotations.minimal_projection = new Ast.Value.Array([]);
        const agg = new Ast.AggregationExpression(null, t, p.name, 'avg', newSchema);
        return new Ast.ProjectionExpression(null, agg, [p.name], [], [], newSchema);
    }), C.expressionKeyFn, {  });
   $grammar.addRule("filtered_table", [new $runtime.NonTerminal("out_param_Any", "p", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.NonTerminal("filtered_table", "t", undefined)], $locale._("average ${p} of ${t}"), ((p : C.ParamSlot, t : Ast.Expression) : (Ast.Expression)|null => {
        const schema = t.schema!;
        if (!schema.out[p.name] || !schema.out[p.name].isNumeric())
            return null;
        if (!schema.is_list)
            return null;

        const newSchema = schema.filterArguments((arg) => arg.direction !== Ast.ArgDirection.OUT || arg.name === p.name);
        newSchema.minimal_projection = [];
        if (newSchema instanceof Ast.FunctionDef)
            newSchema.annotations.minimal_projection = new Ast.Value.Array([]);
        const agg = new Ast.AggregationExpression(null, t, p.name, 'avg', newSchema);
        return new Ast.ProjectionExpression(null, agg, [p.name], [], [], newSchema);
    }), C.expressionKeyFn, {  });
   $grammar.declareSymbol("$root");
   $grammar.addRule("$root", [new $runtime.NonTerminal("table", "t", undefined)], "${t}", ((t : Ast.Expression) : (Ast.Input)|null => {
        const proj = C.sayProjection($loader, t);
        if (proj)
            return C.makeProgram($loader, proj);
        return null;
    }), undefined, {  });
   $grammar.addRule("$root", [new $runtime.NonTerminal("filtered_table", "t", undefined)], "${t}", ((t : Ast.Expression) : (Ast.Input)|null => {
        const filtered = C.sayProjection($loader, t);
        if (filtered)
            return C.makeProgram($loader, filtered);
        return null;
    }), undefined, {  });
};
