// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2021 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

import { Ast, } from 'thingtalk';
import * as C from '../ast_manip';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {
   $grammar.declareSymbol("same_sentence_coref");
   $grammar.addRule("same_sentence_coref", [], "it", (() : (any)|null => "it"), undefined, { identity: (true) });
   $grammar.addRule("same_sentence_coref", [], "them", (() : (any)|null => "them"), undefined, { identity: (true) });
   $grammar.declareSymbol("context_coref");
   $grammar.addRule("context_coref", [], "it", (() : (any)|null => "it"), undefined, { identity: (true) });
   $grammar.addRule("context_coref", [], "this", (() : (any)|null => "this"), undefined, { identity: (true) });
   $grammar.addRule("context_coref", [], "that", (() : (any)|null => "that"), undefined, { identity: (true) });
   $grammar.declareSymbol("list_context_coref");
   $grammar.addRule("list_context_coref", [], "either one", (() : (any)|null => "either one"), undefined, { identity: (true) });
   $grammar.addRule("list_context_coref", [], "either of them", (() : (any)|null => "either of them"), undefined, { identity: (true) });
   $grammar.addRule("list_context_coref", [], "any of them", (() : (any)|null => "any of them"), undefined, { identity: (true) });
   $grammar.addRule("list_context_coref", [], "any one of them", (() : (any)|null => "any one of them"), undefined, { identity: (true) });
   $grammar.declareSymbol("the_base_table");
   $grammar.addRule("the_base_table", [new $runtime.NonTerminal("base_table", "table", undefined)], $locale._("the ${table}"), ((table : Ast.InvocationExpression) : (Ast.InvocationExpression)|null => table), C.expressionKeyFn, {  });
};
