// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2019-2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

import { Ast, } from 'thingtalk';
import * as C from '../ast_manip';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {

$loader = new ThingpediaLoader($runtime, $ttUtils, $grammar, $locale, $options);
   await (await $runtime.import("../common.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);

await $loader.init();
   await (await $runtime.import("./shared.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./constants.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./timers.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./filters.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./projections.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./aggregation.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./computation.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./who_questions.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./commands.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   await (await $runtime.import("./stream_tables.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   $grammar.declareSymbol("$root");
   $grammar.addRule("$root", [new $runtime.NonTerminal("thingpedia_complete_program", "p", undefined)], "${p}", ((p : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, p)), undefined, {  });
   if (!$options.flags.nostream) {
       if ($options.flags.timer) {
           $grammar.addRule("$root", [new $runtime.NonTerminal("timer_rule", "stmt", undefined)], "${stmt}", ((stmt : Ast.ChainExpression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, { weight: (0.2) });
       }
       $grammar.addRule("$root", [new $runtime.NonTerminal("monitor_command", "stmt", undefined)], "${stmt}", ((stmt : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, { weight: (0.2) });
       $grammar.addRule("$root", [new $runtime.NonTerminal("when_command", "stmt", undefined)], "${stmt}", ((stmt : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, { weight: (0.2) });
   }
   $grammar.addRule("$root", [new $runtime.NonTerminal("say_command", "stmt", undefined)], "${stmt}", ((stmt : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, { weight: (0.1) });
   $grammar.addRule("$root", [new $runtime.NonTerminal("complete_action", "stmt", undefined)], "${stmt}", ((stmt : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, {  });
   $grammar.addRule("$root", [new $runtime.NonTerminal("generic_preamble_for_action", undefined, undefined), new $runtime.NonTerminal("complete_action", "stmt", undefined)], $locale._("${generic_preamble_for_action} ${stmt}"), (($0 : string, stmt : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, {  });
   $grammar.addRule("$root", [new $runtime.NonTerminal("generic_preamble_for_action", undefined, undefined), new $runtime.NonTerminal("forward_get_do_command", "stmt", undefined)], $locale._("${generic_preamble_for_action} ${stmt}"), (($0 : string, stmt : Ast.ChainExpression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, {  });
   $grammar.addRule("$root", [new $runtime.NonTerminal("backward_get_do_command", "stmt", undefined)], "${stmt}", ((stmt : Ast.ChainExpression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, { weight: (0.1) });
   $grammar.addRule("$root", [new $runtime.NonTerminal("search_command", "stmt", undefined)], "${stmt}", ((stmt : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, {  });
   $grammar.addRule("$root", [new $runtime.NonTerminal("complete_question", "stmt", undefined)], "${stmt}", ((stmt : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, { weight: (0.1) });
   $grammar.addRule("$root", [new $runtime.NonTerminal("projection_question", "stmt", undefined)], "${stmt}", ((stmt : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stmt)), undefined, { weight: (0.1) });
};
