// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2019-2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

import { Ast, } from 'thingtalk';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {

const TIME_MORNING = new Ast.Value.Time(new Ast.Time.Relative("morning"));
const TIME_NOON = new Ast.Value.Time(new Ast.Time.Absolute(12, 0, 0));
const TIME_EVENING = new Ast.Value.Time(new Ast.Time.Relative("evening"));
   $grammar.declareSymbol("optional_comma");
   $grammar.addRule("optional_comma", [], ",", (() : (any)|null => ','), undefined, {  });
   $grammar.addRule("optional_comma", [], "", (() : (any)|null => ''), undefined, {  });
   $grammar.declareSymbol("per_day");
   $grammar.addRule("per_day", [], "each day", (() : (any)|null => 'each day'), undefined, {  });
   $grammar.addRule("per_day", [], "every day", (() : (any)|null => 'every day'), undefined, {  });
   $grammar.addRule("per_day", [], "daily", (() : (any)|null => 'daily'), undefined, {  });
   $grammar.addRule("per_day", [], "a day", (() : (any)|null => 'a day'), undefined, {  });
   $grammar.addRule("per_day", [], "per day", (() : (any)|null => 'per day'), undefined, {  });
   $grammar.declareSymbol("optional_per_day");
   $grammar.addRule("optional_per_day", [], "each day", (() : (any)|null => 'each day'), undefined, {  });
   $grammar.addRule("optional_per_day", [], "every day", (() : (any)|null => 'every day'), undefined, {  });
   $grammar.addRule("optional_per_day", [], "daily", (() : (any)|null => 'daily'), undefined, {  });
   $grammar.addRule("optional_per_day", [], "a day", (() : (any)|null => 'a day'), undefined, {  });
   $grammar.addRule("optional_per_day", [], "per day", (() : (any)|null => 'per day'), undefined, {  });
   $grammar.addRule("optional_per_day", [], "", (() : (any)|null => ''), undefined, {  });
   $grammar.declareSymbol("once_per_day");
   $grammar.addRule("once_per_day", [], "each day", (() : (any)|null => 'each day'), undefined, {  });
   $grammar.addRule("once_per_day", [], "every day", (() : (any)|null => 'every day'), undefined, {  });
   $grammar.addRule("once_per_day", [], "daily", (() : (any)|null => 'daily'), undefined, {  });
   $grammar.addRule("once_per_day", [], "a day", (() : (any)|null => 'a day'), undefined, {  });
   $grammar.addRule("once_per_day", [], "per day", (() : (any)|null => 'per day'), undefined, {  });
   $grammar.addRule("once_per_day", [], "once a day", (() : (any)|null => 'once a day'), undefined, {  });
   $grammar.addRule("once_per_day", [], "once per day", (() : (any)|null => 'once per day'), undefined, {  });
   $grammar.addRule("once_per_day", [], "once daily", (() : (any)|null => 'once daily'), undefined, {  });
   $grammar.addRule("once_per_day", [], "one time a day", (() : (any)|null => 'one time a day'), undefined, {  });
   $grammar.addRule("once_per_day", [], "one time per day", (() : (any)|null => 'one time per day'), undefined, {  });
   $grammar.addRule("once_per_day", [], "one time daily", (() : (any)|null => 'one time daily'), undefined, {  });
   $grammar.declareSymbol("twice_per_day");
   $grammar.addRule("twice_per_day", [], "twice a day", (() : (any)|null => 'twice a day'), undefined, {  });
   $grammar.addRule("twice_per_day", [], "twice per day", (() : (any)|null => 'twice per day'), undefined, {  });
   $grammar.addRule("twice_per_day", [], "twice daily", (() : (any)|null => 'twice daily'), undefined, {  });
   $grammar.addRule("twice_per_day", [], "two times a day", (() : (any)|null => 'two times a day'), undefined, {  });
   $grammar.addRule("twice_per_day", [], "two times per day", (() : (any)|null => 'two times per day'), undefined, {  });
   $grammar.addRule("twice_per_day", [], "two times daily", (() : (any)|null => 'two times daily'), undefined, {  });
   $grammar.declareSymbol("thrice_per_day");
   $grammar.addRule("thrice_per_day", [], "thrice a day", (() : (any)|null => 'thrice a day'), undefined, {  });
   $grammar.addRule("thrice_per_day", [], "thrice per day", (() : (any)|null => 'thrice per day'), undefined, {  });
   $grammar.addRule("thrice_per_day", [], "thrice daily", (() : (any)|null => 'thrice daily'), undefined, {  });
   $grammar.addRule("thrice_per_day", [], "three times a day", (() : (any)|null => 'three times a day'), undefined, {  });
   $grammar.addRule("thrice_per_day", [], "three times per day", (() : (any)|null => 'three times per day'), undefined, {  });
   $grammar.addRule("thrice_per_day", [], "three times daily", (() : (any)|null => 'three times daily'), undefined, {  });
   $grammar.declareSymbol("optional_once");
   $grammar.addRule("optional_once", [], "once", (() : (any)|null => 'once'), undefined, {  });
   $grammar.addRule("optional_once", [], "one time", (() : (any)|null => 'one time'), undefined, {  });
   $grammar.addRule("optional_once", [], "", (() : (any)|null => ''), undefined, {  });
   $grammar.declareSymbol("optional_twice");
   $grammar.addRule("optional_twice", [], "twice", (() : (any)|null => 'twice'), undefined, {  });
   $grammar.addRule("optional_twice", [], "two times", (() : (any)|null => 'two times'), undefined, {  });
   $grammar.addRule("optional_twice", [], "", (() : (any)|null => ''), undefined, {  });
   $grammar.declareSymbol("each");
   $grammar.addRule("each", [], "each", (() : (any)|null => 'each'), undefined, {  });
   $grammar.addRule("each", [], "every", (() : (any)|null => 'every'), undefined, {  });
   $grammar.addRule("each", [], "per", (() : (any)|null => 'per'), undefined, {  });
   $grammar.addRule("each", [], "a", (() : (any)|null => 'a'), undefined, {  });
   $grammar.declareSymbol("and");
   $grammar.addRule("and", [], "and", (() : (any)|null => 'and'), undefined, {  });
   $grammar.addRule("and", [], "and also", (() : (any)|null => 'and also'), undefined, {  });
   $grammar.addRule("and", [], "plus", (() : (any)|null => 'plus'), undefined, {  });
   $grammar.addRule("and", [], "in addition to", (() : (any)|null => 'in addition to'), undefined, {  });
   $grammar.addRule("and", [], ",", (() : (any)|null => ','), undefined, {  });
   $grammar.declareSymbol("optional_first");
   $grammar.addRule("optional_first", [], "one", (() : (any)|null => 'one'), undefined, {  });
   $grammar.addRule("optional_first", [], "once", (() : (any)|null => 'once'), undefined, {  });
   $grammar.addRule("optional_first", [], "one time", (() : (any)|null => 'one time'), undefined, {  });
   $grammar.addRule("optional_first", [], "first", (() : (any)|null => 'first'), undefined, {  });
   $grammar.addRule("optional_first", [], "the first", (() : (any)|null => 'the first'), undefined, {  });
   $grammar.addRule("optional_first", [], "the first one", (() : (any)|null => 'the first one'), undefined, {  });
   $grammar.addRule("optional_first", [], "", (() : (any)|null => ''), undefined, {  });
   $grammar.declareSymbol("optional_second");
   $grammar.addRule("optional_second", [], "one", (() : (any)|null => 'one'), undefined, {  });
   $grammar.addRule("optional_second", [], "once", (() : (any)|null => 'once'), undefined, {  });
   $grammar.addRule("optional_second", [], "one time", (() : (any)|null => 'one time'), undefined, {  });
   $grammar.addRule("optional_second", [], "one more", (() : (any)|null => 'one more'), undefined, {  });
   $grammar.addRule("optional_second", [], "once more", (() : (any)|null => 'once more'), undefined, {  });
   $grammar.addRule("optional_second", [], "one more time", (() : (any)|null => 'one more time'), undefined, {  });
   $grammar.addRule("optional_second", [], "again", (() : (any)|null => 'again'), undefined, {  });
   $grammar.addRule("optional_second", [], "once again", (() : (any)|null => 'once again'), undefined, {  });
   $grammar.addRule("optional_second", [], "another", (() : (any)|null => 'another'), undefined, {  });
   $grammar.addRule("optional_second", [], "another time", (() : (any)|null => 'another time'), undefined, {  });
   $grammar.addRule("optional_second", [], "another one", (() : (any)|null => 'another one'), undefined, {  });
   $grammar.addRule("optional_second", [], "second", (() : (any)|null => 'second'), undefined, {  });
   $grammar.addRule("optional_second", [], "a second", (() : (any)|null => 'a second'), undefined, {  });
   $grammar.addRule("optional_second", [], "a second one", (() : (any)|null => 'a second one'), undefined, {  });
   $grammar.addRule("optional_second", [], "the second", (() : (any)|null => 'the second'), undefined, {  });
   $grammar.addRule("optional_second", [], "the second one", (() : (any)|null => 'the second one'), undefined, {  });
   $grammar.addRule("optional_second", [], "", (() : (any)|null => ''), undefined, {  });
   $grammar.declareSymbol("optional_emphasis");
   $grammar.addRule("optional_emphasis", [], "just", (() : (any)|null => 'just'), undefined, {  });
   $grammar.addRule("optional_emphasis", [], "right", (() : (any)|null => 'right'), undefined, {  });
   $grammar.addRule("optional_emphasis", [], "", (() : (any)|null => ''), undefined, {  });
   $grammar.declareSymbol("at_noon");
   $grammar.addRule("at_noon", [], "at noon", (() : (any)|null => 'at noon'), undefined, {  });
   $grammar.addRule("at_noon", [], "in the middle of the day", (() : (any)|null => 'in the middle of the day'), undefined, {  });
   $grammar.addRule("at_noon", [], "at lunch", (() : (any)|null => 'at lunch'), undefined, {  });
   $grammar.declareSymbol("at_morning");
   $grammar.addRule("at_morning", [], "when i get up", (() : (any)|null => 'when i get up'), undefined, {  });
   $grammar.addRule("at_morning", [], "when i wake up", (() : (any)|null => 'when i wake up'), undefined, {  });
   $grammar.addRule("at_morning", [], "after getting up", (() : (any)|null => 'after getting up'), undefined, {  });
   $grammar.addRule("at_morning", [], "after i get up", (() : (any)|null => 'after i get up'), undefined, {  });
   $grammar.addRule("at_morning", [], "after waking up", (() : (any)|null => 'after waking up'), undefined, {  });
   $grammar.addRule("at_morning", [], "after i wake up", (() : (any)|null => 'after i wake up'), undefined, {  });
   $grammar.addRule("at_morning", [], "during the morning", (() : (any)|null => 'during the morning'), undefined, {  });
   $grammar.addRule("at_morning", [], "each morning", (() : (any)|null => 'each morning'), undefined, {  });
   $grammar.addRule("at_morning", [], "every morning", (() : (any)|null => 'every morning'), undefined, {  });
   $grammar.addRule("at_morning", [], "in the morning", (() : (any)|null => 'in the morning'), undefined, {  });
   $grammar.addRule("at_morning", [], "at breakfast", (() : (any)|null => 'at breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "with breakfast", (() : (any)|null => 'with breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "during breakfast", (() : (any)|null => 'during breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "before breakfast", (() : (any)|null => 'before breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "before eating breakfast", (() : (any)|null => 'before eating breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "before i eat breakfast", (() : (any)|null => 'before i eat breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "after breakfast", (() : (any)|null => 'after breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "after eating breakfast", (() : (any)|null => 'after eating breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "after i eat breakfast", (() : (any)|null => 'after i eat breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "when i eat breakfast", (() : (any)|null => 'when i eat breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "while i eat breakfast", (() : (any)|null => 'while i eat breakfast'), undefined, {  });
   $grammar.addRule("at_morning", [], "before school", (() : (any)|null => 'before school'), undefined, {  });
   $grammar.addRule("at_morning", [], "before going to school", (() : (any)|null => 'before going to school'), undefined, {  });
   $grammar.addRule("at_morning", [], "before i go to school", (() : (any)|null => 'before i go to school'), undefined, {  });
   $grammar.addRule("at_morning", [], "before leaving for school", (() : (any)|null => 'before leaving for school'), undefined, {  });
   $grammar.addRule("at_morning", [], "before i leave for school", (() : (any)|null => 'before i leave for school'), undefined, {  });
   $grammar.addRule("at_morning", [], "before heading to school", (() : (any)|null => 'before heading to school'), undefined, {  });
   $grammar.addRule("at_morning", [], "before i head to school", (() : (any)|null => 'before i head to school'), undefined, {  });
   $grammar.addRule("at_morning", [], "when i go to school", (() : (any)|null => 'when i go to school'), undefined, {  });
   $grammar.addRule("at_morning", [], "when i leave for school", (() : (any)|null => 'when i leave for school'), undefined, {  });
   $grammar.addRule("at_morning", [], "when i head to school", (() : (any)|null => 'when i head to school'), undefined, {  });
   $grammar.addRule("at_morning", [], "before work", (() : (any)|null => 'before work'), undefined, {  });
   $grammar.addRule("at_morning", [], "before going to work", (() : (any)|null => 'before going to work'), undefined, {  });
   $grammar.addRule("at_morning", [], "before i go to work", (() : (any)|null => 'before i go to work'), undefined, {  });
   $grammar.addRule("at_morning", [], "before leaving for work", (() : (any)|null => 'before leaving for work'), undefined, {  });
   $grammar.addRule("at_morning", [], "before i leave for work", (() : (any)|null => 'before i leave for work'), undefined, {  });
   $grammar.addRule("at_morning", [], "before heading to work", (() : (any)|null => 'before heading to work'), undefined, {  });
   $grammar.addRule("at_morning", [], "before i head to work", (() : (any)|null => 'before i head to work'), undefined, {  });
   $grammar.addRule("at_morning", [], "when i go to work", (() : (any)|null => 'when i go to work'), undefined, {  });
   $grammar.addRule("at_morning", [], "when i leave for work", (() : (any)|null => 'when i leave for work'), undefined, {  });
   $grammar.addRule("at_morning", [], "when i head to work", (() : (any)|null => 'when i head to work'), undefined, {  });
   $grammar.declareSymbol("at_evening");
   $grammar.addRule("at_evening", [], "during the evening", (() : (any)|null => 'during the evening'), undefined, {  });
   $grammar.addRule("at_evening", [], "each evening", (() : (any)|null => 'each evening'), undefined, {  });
   $grammar.addRule("at_evening", [], "every evening", (() : (any)|null => 'every evening'), undefined, {  });
   $grammar.addRule("at_evening", [], "in the evening", (() : (any)|null => 'in the evening'), undefined, {  });
   $grammar.addRule("at_evening", [], "during the night", (() : (any)|null => 'during the night'), undefined, {  });
   $grammar.addRule("at_evening", [], "each night", (() : (any)|null => 'each night'), undefined, {  });
   $grammar.addRule("at_evening", [], "every night", (() : (any)|null => 'every night'), undefined, {  });
   $grammar.addRule("at_evening", [], "in the night", (() : (any)|null => 'in the night'), undefined, {  });
   $grammar.addRule("at_evening", [], "at night", (() : (any)|null => 'at night'), undefined, {  });
   $grammar.addRule("at_evening", [], "at dinner", (() : (any)|null => 'at dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "with dinner", (() : (any)|null => 'with dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "during dinner", (() : (any)|null => 'during dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "before dinner", (() : (any)|null => 'before dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "before eating dinner", (() : (any)|null => 'before eating dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "before i eat dinner", (() : (any)|null => 'before i eat dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "after dinner", (() : (any)|null => 'after dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "after eating dinner", (() : (any)|null => 'after eating dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "after i eat dinner", (() : (any)|null => 'after i eat dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i eat dinner", (() : (any)|null => 'when i eat dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "while i eat dinner", (() : (any)|null => 'while i eat dinner'), undefined, {  });
   $grammar.addRule("at_evening", [], "after school", (() : (any)|null => 'after school'), undefined, {  });
   $grammar.addRule("at_evening", [], "after getting back from school", (() : (any)|null => 'after getting back from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "after returning from school", (() : (any)|null => 'after returning from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "after coming home from school", (() : (any)|null => 'after coming home from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "after i get back from school", (() : (any)|null => 'after i get back from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "after i return from school", (() : (any)|null => 'after i return from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "after i come home from school", (() : (any)|null => 'after i come home from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i get back from school", (() : (any)|null => 'when i get back from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i return from school", (() : (any)|null => 'when i return from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i come home from school", (() : (any)|null => 'when i come home from school'), undefined, {  });
   $grammar.addRule("at_evening", [], "after work", (() : (any)|null => 'after work'), undefined, {  });
   $grammar.addRule("at_evening", [], "after getting back from work", (() : (any)|null => 'after getting back from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "after returning from work", (() : (any)|null => 'after returning from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "after coming home from work", (() : (any)|null => 'after coming home from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "after i get back from work", (() : (any)|null => 'after i get back from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "after i return from work", (() : (any)|null => 'after i return from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "after i come home from work", (() : (any)|null => 'after i come home from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i get back from work", (() : (any)|null => 'when i get back from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i return from work", (() : (any)|null => 'when i return from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i come home from work", (() : (any)|null => 'when i come home from work'), undefined, {  });
   $grammar.addRule("at_evening", [], "before bed", (() : (any)|null => 'before bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "before going to bed", (() : (any)|null => 'before going to bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "before i go to bed", (() : (any)|null => 'before i go to bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "before leaving for bed", (() : (any)|null => 'before leaving for bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "before i leave for bed", (() : (any)|null => 'before i leave for bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "before heading to bed", (() : (any)|null => 'before heading to bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "before i head to bed", (() : (any)|null => 'before i head to bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i go to bed", (() : (any)|null => 'when i go to bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i leave for bed", (() : (any)|null => 'when i leave for bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "when i head to bed", (() : (any)|null => 'when i head to bed'), undefined, {  });
   $grammar.addRule("at_evening", [], "at bedtime", (() : (any)|null => 'at bedtime'), undefined, {  });
   $grammar.addRule("at_evening", [], "before bedtime", (() : (any)|null => 'before bedtime'), undefined, {  });
   $grammar.addRule("at_evening", [], "before i go to sleep", (() : (any)|null => 'before i go to sleep'), undefined, {  });
   $grammar.addRule("at_evening", [], "before going to sleep", (() : (any)|null => 'before going to sleep'), undefined, {  });
   $grammar.addRule("at_evening", [], "before sleeping", (() : (any)|null => 'before sleeping'), undefined, {  });
   $grammar.addRule("at_evening", [], "before i sleep", (() : (any)|null => 'before i sleep'), undefined, {  });
   $grammar.addRule("at_evening", [], "before i fall asleep", (() : (any)|null => 'before i fall asleep'), undefined, {  });
   $grammar.addRule("at_evening", [], "before falling asleep", (() : (any)|null => 'before falling asleep'), undefined, {  });
   $grammar.declareSymbol("at_evening_second");
   $grammar.addRule("at_evening_second", [], "evening", (() : (any)|null => 'evening'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "work", (() : (any)|null => 'work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "school", (() : (any)|null => 'school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "dinner", (() : (any)|null => 'dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "night", (() : (any)|null => 'night'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "bed", (() : (any)|null => 'bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "during the evening", (() : (any)|null => 'during the evening'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "each evening", (() : (any)|null => 'each evening'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "every evening", (() : (any)|null => 'every evening'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "in the evening", (() : (any)|null => 'in the evening'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "during the night", (() : (any)|null => 'during the night'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "each night", (() : (any)|null => 'each night'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "every night", (() : (any)|null => 'every night'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "in the night", (() : (any)|null => 'in the night'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "at night", (() : (any)|null => 'at night'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "at dinner", (() : (any)|null => 'at dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "with dinner", (() : (any)|null => 'with dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "during dinner", (() : (any)|null => 'during dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before dinner", (() : (any)|null => 'before dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before eating dinner", (() : (any)|null => 'before eating dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before i eat dinner", (() : (any)|null => 'before i eat dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after dinner", (() : (any)|null => 'after dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after eating dinner", (() : (any)|null => 'after eating dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after i eat dinner", (() : (any)|null => 'after i eat dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i eat dinner", (() : (any)|null => 'when i eat dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "while i eat dinner", (() : (any)|null => 'while i eat dinner'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after school", (() : (any)|null => 'after school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after getting back from school", (() : (any)|null => 'after getting back from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after returning from school", (() : (any)|null => 'after returning from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after coming home from school", (() : (any)|null => 'after coming home from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after i get back from school", (() : (any)|null => 'after i get back from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after i return from school", (() : (any)|null => 'after i return from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after i come home from school", (() : (any)|null => 'after i come home from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i get back from school", (() : (any)|null => 'when i get back from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i return from school", (() : (any)|null => 'when i return from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i come home from school", (() : (any)|null => 'when i come home from school'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after work", (() : (any)|null => 'after work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after getting back from work", (() : (any)|null => 'after getting back from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after returning from work", (() : (any)|null => 'after returning from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after coming home from work", (() : (any)|null => 'after coming home from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after i get back from work", (() : (any)|null => 'after i get back from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after i return from work", (() : (any)|null => 'after i return from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "after i come home from work", (() : (any)|null => 'after i come home from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i get back from work", (() : (any)|null => 'when i get back from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i return from work", (() : (any)|null => 'when i return from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i come home from work", (() : (any)|null => 'when i come home from work'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before bed", (() : (any)|null => 'before bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before going to bed", (() : (any)|null => 'before going to bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before i go to bed", (() : (any)|null => 'before i go to bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before leaving for bed", (() : (any)|null => 'before leaving for bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before i leave for bed", (() : (any)|null => 'before i leave for bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before heading to bed", (() : (any)|null => 'before heading to bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before i head to bed", (() : (any)|null => 'before i head to bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i go to bed", (() : (any)|null => 'when i go to bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i leave for bed", (() : (any)|null => 'when i leave for bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "when i head to bed", (() : (any)|null => 'when i head to bed'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "at bedtime", (() : (any)|null => 'at bedtime'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before bedtime", (() : (any)|null => 'before bedtime'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before i go to sleep", (() : (any)|null => 'before i go to sleep'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before going to sleep", (() : (any)|null => 'before going to sleep'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before sleeping", (() : (any)|null => 'before sleeping'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before i sleep", (() : (any)|null => 'before i sleep'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before i fall asleep", (() : (any)|null => 'before i fall asleep'), undefined, {  });
   $grammar.addRule("at_evening_second", [], "before falling asleep", (() : (any)|null => 'before falling asleep'), undefined, {  });
   $grammar.declareSymbol("when_brushing_teeth");
   $grammar.addRule("when_brushing_teeth", [], "when i brush my teeth", (() : (any)|null => 'when i brush my teeth'), undefined, {  });
   $grammar.addRule("when_brushing_teeth", [], "when brushing my teeth", (() : (any)|null => 'when brushing my teeth'), undefined, {  });
   $grammar.addRule("when_brushing_teeth", [], "before brushing my teeth", (() : (any)|null => 'before brushing my teeth'), undefined, {  });
   $grammar.addRule("when_brushing_teeth", [], "before i brush my teeth", (() : (any)|null => 'before i brush my teeth'), undefined, {  });
   $grammar.addRule("when_brushing_teeth", [], "after brushing my teeth", (() : (any)|null => 'after brushing my teeth'), undefined, {  });
   $grammar.addRule("when_brushing_teeth", [], "after i brush my teeth", (() : (any)|null => 'after i brush my teeth'), undefined, {  });
   $grammar.declareSymbol("on");
   $grammar.addRule("on", [], "on", (() : (any)|null => 'on'), undefined, {  });
   $grammar.addRule("on", [], "when it 's", (() : (any)|null => 'when it \'s'), undefined, {  });
   $grammar.addRule("on", [], "when it is", (() : (any)|null => 'when it is'), undefined, {  });
   $grammar.declareSymbol("timer");
   if ($options.flags.extended_timers) {
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_once", undefined, undefined), new $runtime.NonTerminal("optional_emphasis", undefined, undefined), new $runtime.NonTerminal("at_noon", undefined, undefined)], $locale._("${optional_once} ${optional_emphasis} ${at_noon}"), (($0 : any, $1 : any, $2 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.AtTimer(null, [TIME_NOON], null, $loader.standardSchemas.attimer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_once", undefined, undefined), new $runtime.NonTerminal("optional_emphasis", undefined, undefined), new $runtime.NonTerminal("at_morning", undefined, undefined)], $locale._("${optional_once} ${optional_emphasis} ${at_morning}"), (($0 : any, $1 : any, $2 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.AtTimer(null, [TIME_MORNING], null, $loader.standardSchemas.attimer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_once", undefined, undefined), new $runtime.NonTerminal("optional_emphasis", undefined, undefined), new $runtime.NonTerminal("at_evening", undefined, undefined)], $locale._("${optional_once} ${optional_emphasis} ${at_evening}"), (($0 : any, $1 : any, $2 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.AtTimer(null, [TIME_EVENING], null, $loader.standardSchemas.attimer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_twice", undefined, undefined), new $runtime.NonTerminal("optional_per_day", undefined, undefined), new $runtime.NonTerminal("optional_comma", undefined, undefined), new $runtime.NonTerminal("optional_first", undefined, undefined), new $runtime.NonTerminal("optional_emphasis", undefined, undefined), new $runtime.NonTerminal("at_morning", undefined, undefined), new $runtime.NonTerminal("and", undefined, undefined), new $runtime.NonTerminal("optional_second", undefined, undefined), new $runtime.NonTerminal("optional_emphasis", undefined, undefined), new $runtime.NonTerminal("at_evening_second", undefined, undefined)], $locale._("${optional_twice} ${optional_per_day} ${optional_comma} ${optional_first} ${optional_emphasis} ${at_morning} ${and} ${optional_second} ${optional_emphasis} ${at_evening_second}"), (($0 : any, $1 : any, $2 : any, $3 : any, $4 : any, $5 : any, $6 : any, $7 : any, $8 : any, $9 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.AtTimer(null, [TIME_MORNING, TIME_EVENING], null, $loader.standardSchemas.attimer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("when_brushing_teeth", undefined, undefined)], "${when_brushing_teeth}", (($0 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.AtTimer(null, [TIME_MORNING, TIME_EVENING], null, $loader.standardSchemas.attimer).toExpression()), undefined, {  });
   }
   $grammar.addRule("timer", [new $runtime.NonTerminal("once_per_day", undefined, undefined), new $runtime.NonTerminal("constant_Time", "time", undefined)], $locale._("${once_per_day} at ${time}"), (($0 : any, time : Ast.Value) : (Ast.FunctionCallExpression)|null => new Ast.Stream.AtTimer(null, [time], null, $loader.standardSchemas.attimer).toExpression()), undefined, {  });
   $grammar.addRule("timer", [new $runtime.NonTerminal("twice_per_day", undefined, undefined), new $runtime.NonTerminal("constant_Time", "time1", undefined), new $runtime.NonTerminal("and", undefined, undefined), new $runtime.NonTerminal("constant_Time", "time2", undefined)], $locale._("${twice_per_day} at ${time1} ${and} ${time2}"), (($0 : any, time1 : Ast.Value, $1 : any, time2 : Ast.Value) : (Ast.FunctionCallExpression)|null => new Ast.Stream.AtTimer(null, [time1, time2], null, $loader.standardSchemas.attimer).toExpression()), undefined, {  });
   $grammar.addRule("timer", [new $runtime.NonTerminal("thrice_per_day", undefined, undefined), new $runtime.NonTerminal("constant_Time", "time1", undefined), new $runtime.NonTerminal("and", undefined, undefined), new $runtime.NonTerminal("constant_Time", "time2", undefined), new $runtime.NonTerminal("and", undefined, undefined), new $runtime.NonTerminal("constant_Time", "time3", undefined)], $locale._("${thrice_per_day} at ${time1} ${and} ${time2} ${and} ${time3}"), (($0 : any, time1 : Ast.Value, $1 : any, time2 : Ast.Value, $2 : any, time3 : Ast.Value) : (Ast.FunctionCallExpression)|null => new Ast.Stream.AtTimer(null, [time1, time2, time3], null, $loader.standardSchemas.attimer).toExpression()), undefined, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_once", undefined, undefined), new $runtime.NonTerminal("each", undefined, undefined), new $runtime.NonTerminal("constant_Measure_ms", "interval", undefined)], $locale._("${optional_once} ${each} ${interval}"), (($0 : any, $1 : any, interval : Ast.Value) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), interval, null, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("once_per_day", undefined, undefined)], "${once_per_day}", (($0 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'day'), null, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_once", undefined, undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${optional_once} ${each} year"), (($0 : any, $1 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'year'), null, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_once", undefined, undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${optional_once} ${each} month"), (($0 : any, $1 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'mon'), null, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_once", undefined, undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${optional_once} ${each} week"), (($0 : any, $1 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'week'), null, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("optional_once", undefined, undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${optional_once} ${each} hour"), (($0 : any, $1 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'h'), null, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("constant_Number", "freq", undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${freq} times ${each} minute"), ((freq : Ast.NumberValue, $0 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'min'), freq, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("constant_Number", "freq", undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${freq} times ${each} hour"), ((freq : Ast.NumberValue, $0 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'h'), freq, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("constant_Number", "freq", undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${freq} times ${each} day"), ((freq : Ast.NumberValue, $0 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'day'), freq, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("constant_Number", "freq", undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${freq} times ${each} week"), ((freq : Ast.NumberValue, $0 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'week'), freq, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("constant_Number", "freq", undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${freq} times ${each} month"), ((freq : Ast.NumberValue, $0 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'mon'), freq, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("constant_Number", "freq", undefined), new $runtime.NonTerminal("each", undefined, undefined)], $locale._("${freq} times ${each} year"), ((freq : Ast.NumberValue, $0 : any) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), new Ast.Value.Measure(1, 'year'), freq, $loader.standardSchemas.timer).toExpression()), undefined, {  });
       $grammar.addRule("timer", [new $runtime.NonTerminal("constant_Number", "freq", undefined), new $runtime.NonTerminal("each", undefined, undefined), new $runtime.NonTerminal("constant_Measure_ms", "interval", undefined)], $locale._("${freq} times ${each} ${interval}"), ((freq : Ast.NumberValue, $0 : any, interval : Ast.Value) : (Ast.FunctionCallExpression)|null => new Ast.Stream.Timer(null, new Ast.Value.Date(null), interval, freq, $loader.standardSchemas.timer).toExpression()), undefined, {  });
   }
};
